/** 
 * 
 */
package com.chinaitop.depot.basic.controller;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.basic.model.BasicCodeRule;
import com.chinaitop.depot.basic.model.BasicCodeRuleDetail;
import com.chinaitop.depot.basic.service.BasicCodeRuleService;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 * Title: BasicCodeRuleController
 * </p>
 * <p>
 * Description:
 * </p>
 * 
 * @author yaoyabin
 * @date 2017年12月15日 上午10:58:45
 */
@RestController
@RequestMapping(value = "/depot/basic/codeRule")
@Api(value = "BasicCodeRuleController", description = "编码规则主表控制类")
public class BasicCodeRuleController {
	@Resource
	private BasicCodeRuleService basicCodeRuleService;

	@RequestMapping(value = "/getList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value = "编码规则主表数据列表查询", notes = "编码规则主表数据列表查询，支持分页")
	@ApiImplicitParams({ 
		@ApiImplicitParam(name = "pageNum", value = "页数", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页显示条数", paramType = "query"),
		@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query") })
	public PageInfo<BasicCodeRule> getList(Integer pageNum, Integer pageSize, Integer orgId) {
		PageInfo<BasicCodeRule> pageInfo = null;
		try {
			pageInfo = basicCodeRuleService.getPageList(pageNum, pageSize, orgId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return pageInfo;
	}

	@RequestMapping(value = "/getDetailList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value = "编码规则子表数据列表查询", notes = "编码规则子表数据列表查询，支持分页")
	@ApiImplicitParams({ @ApiImplicitParam(name = "pageNum", value = "页数", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页显示条数", paramType = "query"),
			@ApiImplicitParam(name = "bid", value = "主表ID", paramType = "query") })
	public PageInfo<BasicCodeRuleDetail> getDetailList(Integer pageNum, Integer pageSize, Integer bid) {
		PageInfo<BasicCodeRuleDetail> pageInfo = null;
		try {
			pageInfo = basicCodeRuleService.getPageListBasicCodeRuleDetail(pageNum, pageSize, bid);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return pageInfo;
	}

	/**
	 * 根据id查询.
	 * 
	 * @param id 主键
	 * @return
	 */
	@RequestMapping(value = "/edit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value = "根据id查询", notes = "根据id查询")
	@ApiImplicitParams({ @ApiImplicitParam(name = "id", value = "主表ID", paramType = "query") })
	public Map<String, Object> edit(Integer id) {
		Map<String, Object> returnMap = null;
		try {
			if (id != null) {
				returnMap = basicCodeRuleService.findByIdConditionMap(id);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return returnMap;
	}

	/**
	 * 数据更新
	 * 
	 * @param codeRuleJson
	 *            主表信息
	 * @param codeRuleDetailJson
	 *            子表信息
	 * @return
	 * @throws JsonParseException
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	@RequestMapping(value = "/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
	@ApiOperation(value = "数据更新", notes = "数据更新")
	@ApiImplicitParams({ @ApiImplicitParam(name = "codeRuleJson", value = "主表数据", paramType = "form"),
			@ApiImplicitParam(name = "codeRuleDetailJson", value = "子表数据", paramType = "form"),
			@ApiImplicitParam(name = "userInfo", value = "登录对象数据", paramType = "form") })
	public Map<String, Object> save(String codeRuleJson, String codeRuleDetailJson, String userInfo)
			throws JsonParseException, JsonMappingException, IOException {
		Map<String, Object> modelMap = null;
		try {
			modelMap = basicCodeRuleService.updateOrSave(codeRuleJson, codeRuleDetailJson, userInfo);
		} catch (Exception e) {
			e.printStackTrace();
			modelMap.put("status", "error");
			modelMap.put("msg", "保存失败！");
			return modelMap;
		}
		return modelMap;
	}

	/**
	 * 删除(同时删除流程)
	 * 
	 * @param id
	 *            主键
	 * @return
	 */
	@RequestMapping(value = "/remove", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
	@ApiOperation(value = "删除(同时删除流程)", notes = "删除(同时删除流程)")
	@ApiImplicitParams({ @ApiImplicitParam(name = "id", value = "主键", paramType = "form") })
	public Map<String, Object> remove(Integer id) {
		Map<String, Object> modelMap = null;
		try {
			if (id != null) {
				modelMap = basicCodeRuleService.removeCodeRuleAll(id);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return modelMap;
	}

	/**
	 * 根据编码类型获取编码.
	 * 
	 * @param request
	 * @param codeRuleType 编码类型.
	 * @return
	 */
	@RequestMapping(value = "/getCodeValueByType", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value = "根据编码类型获取编码", notes = "根据编码类型获取编码")
	@ApiImplicitParams({ @ApiImplicitParam(name = "codeRuleType", value = "编码类型", paramType = "query"),
			@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query") })
	public Map<String, Object> getCodeValueByType(String codeRuleType, Integer orgId ,HttpServletRequest request) {

		//session中获取orgId 页面刷新时，保证orgId有值
		if(orgId == null ){
			 orgId = (Integer) request.getSession().getAttribute("orgId");
		}
		// 返回map.
		Map<String, Object> modelMap = new HashMap<String, Object>();

		try {
			modelMap = basicCodeRuleService.getCodeValue(codeRuleType, orgId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return modelMap;
	}
}
