package com.chinaitop.depot.basic.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.basic.mapper.BasicKeeperMapper;
import com.chinaitop.depot.basic.model.BasicKeeper;
import com.chinaitop.depot.basic.service.BasicEnumService;
import com.chinaitop.depot.basic.service.BasicKeeperEducationService;
import com.chinaitop.depot.basic.service.BasicKeeperHouseService;
import com.chinaitop.depot.basic.service.BasicKeeperService;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 保管员信息管理Controller，保管员信息管理菜单中所有的功能都在这里，包括给保管员分配仓房
 *
 * @author User
 *
 */
@RestController
@RequestMapping(value = "Keeper")
@Api(value= "BasicKeeperController", description = "员工信息控制类")
public class BasicKeeperController {

	@Resource
	private BasicKeeperService keeperService;

	@Resource
	private BasicKeeperEducationService keperEduationService;

	@Resource
	private BasicEnumService enumService;

	@Resource
	private BasicKeeperHouseService keeperHouseService;

	@Resource
	private BasicKeeperMapper keeperMapper;

	@RequestMapping(value = "/getList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查询员工数据列表", notes = "查询员工数据列表，支持分页")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
			@ApiImplicitParam(name = "name", value = "员工姓名", paramType = "query"),
			@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query"),
			@ApiImplicitParam(name = "userId", value = "用户ID", paramType = "query")
	})
	public PageInfo<BasicKeeper> getList(Integer pageNum, Integer pageSize, String keeperName, Integer orgId, Integer userId) {
		PageInfo<BasicKeeper> pageInfo = null;
		try {
			pageInfo = keeperService.getPageList(pageNum, pageSize, keeperName, orgId, userId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return pageInfo;
	}

	@RequestMapping(value = "/getKeeper", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="通过单位ID，获取保管员数据组(保管员数据对外接口)", notes = "通过单位ID，获取保管员数据组(保管员数据对外接口)")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query")
	})
	public List<BasicKeeper> getKeeper(Integer orgId) {
		List<BasicKeeper> list = null;
		try {
			list = keeperService.getKeeperList(orgId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}

	/**
	 * 查询一个保管员信息(包括其它证书信息)
	 *
	 * @param id 保管员数据ID
	 * @return
	 */
	@RequestMapping(value = "/findBasicKeeper", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查询一个员工信息(包括其它证书信息)", notes = "查询一个员工信息(包括其它证书信息)")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "id", value = "员工数据ID", paramType = "query")
	})
	public Map<String, Object> findBasicKeeper(Integer id) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		modelMap = keeperService.findByKeeperAll(id);
		return modelMap;
	}

	/**
	 * 查询一个保管员基础信息 (不包括其它证书信息)
	 *
	 * @param id 保管员数据ID
	 * @return
	 */
	@RequestMapping(value = "/getBasicKeeper", produces = MediaType.APPLICATION_JSON_VALUE)
	@ApiOperation(value="查询一个员工基础信息 (不包括其它证书信息)", notes = "查询一个员工信息(不包括其它证书信息)")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "id", value = "员工数据ID", paramType = "query")
	})
	public BasicKeeper getBasicKeeper(Integer id) {
		BasicKeeper keeper = keeperService.findByIdParmartKey(id);
		return keeper;
	}

	/**
	 * 更新保管员信息
	 *
	 * @param keeper 保管员数据对象
	 * @param xwxlJson 保管员学历数据对象
	 * @param zyjsJson 保管员职业技术等级数据对象
	 * @param zyzgJson 保管员专业资格数据对象
	 * @param sgzJson 保管员上岗证数据对象
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/edit", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="更新员工信息", notes = "更新员工信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "keeper", value = "员工数据对象", paramType = "form"),
			@ApiImplicitParam(name = "xwxlJson", value = "员工学历数据对象", paramType = "form"),
			@ApiImplicitParam(name = "zyjsJson", value = "员工职业技术等级数据对象", paramType = "form"),
			@ApiImplicitParam(name = "zyzgJson", value = "员工专业资格数据对象", paramType = "form"),
			@ApiImplicitParam(name = "sgzJson", value = "员工上岗证数据对象", paramType = "form"),
			@ApiImplicitParam(name = "userInfo", value = "登录信息数据对象", paramType = "form")
	})
	public Map<String, Object> edit(String keeper, String xwxlJson,
									String zyjsJson, String zyzgJson, String sgzJson, String userInfo) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		try {
			modelMap = keeperService.updateOrSave(keeper, xwxlJson, zyjsJson, zyzgJson, sgzJson, userInfo);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return modelMap;
	}

	/**
	 * 逻辑删除保管员信息
	 *
	 * @param id 保管员数据ID
	 * @return
	 */
	@RequestMapping(value = "/remove", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="逻辑删除员工信息", notes = "逻辑删除员工信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "id", value = "员工ID", paramType = "form")
	})
	public Map<String, Object> remove(Integer id, Integer orgId) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		try {
			if (null != id) {
				modelMap = keeperService.removeKeep(id, orgId);
			}
		} catch (Exception e) {
			e.printStackTrace();
			modelMap.put("status", "error");
			modelMap.put("msg", "删除失败！");
		}

		return modelMap;
	}

	/**
	 * 获取当前保管员所在粮库的所有仓房和当前保管员的权限
	 *
	 * @param id 保管员ID
	 * @return
	 */
	@RequestMapping(value = "/getUnitStoreKeepList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取当前保管员所在粮库的所有仓房、油罐和当前保管员的权限", notes = "获取当前保管员所在粮库的所有仓房和当前保管员的权限")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query"),
			@ApiImplicitParam(name = "id", value = "员工ID", paramType = "query"),
			@ApiImplicitParam(name = "libraryType", value = "库类型，0.主粮库，1.代储点粮库", paramType = "query")
	})
	public Map<String, Object> getUnitStoreKeepList(Integer orgId, Integer id, String libraryType) {
		Map<String, Object> map = new HashMap<String, Object>();

		map = keeperService.getUnitStoreKeepList(orgId, id, libraryType);

		return map;
	}

	@RequestMapping(value = "/saveKeeperHouse", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="提交保管员权限分配", notes = "提交保管员权限分配")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "selected", value = "仓房权限数组(当前保管员可管理的仓房)", paramType = "form"),
			@ApiImplicitParam(name = "userInfo", value = "登录数据对象", paramType = "form"),
			@ApiImplicitParam(name = "bgyId", value = "保管员ID", paramType = "form")
	})
	public Map<String, Object> saveKeeperHouse(String selected, String userInfo, Integer bgyId) {
		Map<String, Object> map = new HashMap<String, Object>();

		try {
			keeperService.saveKeeperHouse(selected, bgyId, userInfo);
			map.put("status", "success");
		} catch (Exception e) {
			e.printStackTrace();
			map.put("status", "error");
			map.put("msg", "删除失败！");
		}
		return map;
	}

	/**
	 * 删除一条保管员证件信息
	 *
	 * @param id 证件数据ID
	 * @return
	 */
	@RequestMapping(value = "/removeEducation", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="删除一条证件信息", notes = "提交保管员权限分配")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "id", value = "证件数据ID", paramType = "form")
	})
	public Map<String, Object> removeEducation(Integer id) {
		Map<String, Object> map = new HashMap<String, Object>();

		try {
			keperEduationService.delete(id);
			map.put("status", "success");
		} catch (Exception e) {
			e.printStackTrace();
			map.put("status", "error");
			map.put("msg", "删除失败！");
		}
		return map;
	}

	@RequestMapping(value = "getKepperByHouseId", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="根据仓房id获取保管员", notes = "根据仓房id获取保管员")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "houseId", value = "仓房ID", paramType = "query")
	})
	public List<BasicKeeper> getKepperByHouseId(String houseId) {
		return keeperService.getKepperByHouseId(houseId);
	}

	@RequestMapping(value = "/receivedUserByOrgId", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查询当前保管员所在库中的所有保管员（排除掉当前登录的保管员用户）", notes = "查询当前保管员所在库中的所有保管员（排除掉当前登录的保管员用户）")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userInfo", value = "登录数据对象JSON", paramType = "query"),
			@ApiImplicitParam(name = "roleId", value = "角色ID", paramType = "query")
	})
	public List<BasicKeeper> receivedUserByOrgId(String userInfo,Integer roleId) {
		List<BasicKeeper> list = null;
		try {
			list = keeperService.qtKeeper(userInfo, roleId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}

	@RequestMapping(value = "/getKeeperByLoginUser", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查询当前登录用户所对应的保管员信息", notes = "查询当前登录用户所对应的保管员信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户ID", paramType = "query"),
			@ApiImplicitParam(name = "roleId", value = "保管员ID", paramType = "query")
	})
	public BasicKeeper getKeeperByLoginUser(Integer userId,Integer roleId) {
		BasicKeeper keeper = new BasicKeeper();
		try {
			keeper = keeperService.getKeeperByLoginUser(userId, roleId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return keeper;
	}

	@RequestMapping(value = "/getKeeperByBarn", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查询当前登录的保管员所管理的仓房", notes = "查询当前登录的保管员所管理的仓房")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userInfo", value = "登录数据对象JSON", paramType = "query")
	})
	public List<BasicKeeper> getKeeperByBarn(String userInfo) {
		List<BasicKeeper> keeperList = null;
		try {
			keeperList = keeperService.findBarn(userInfo);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return keeperList;
	}

	@RequestMapping(value="/getKeeperNamesByHouseId", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查询当前仓所对应的保管员", notes = "查询当前仓所对应的保管员")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "houseId", value = "仓房ID", paramType = "form")
	})
	public BasicKeeper getKeeperNamesByHouseId(String houseId){
		BasicKeeper keeper = new BasicKeeper();
		try {
			keeper = keeperService.getKeeperNamesByHouseId(houseId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return keeper;
	}

	@RequestMapping(value = "/getKeeperList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查询保管员数据列表", notes = "查询保管员数据列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNum", value = "页数", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页显示的条数", paramType = "query"),
			@ApiImplicitParam(name = "keeperName", value = "保管员姓名", paramType = "query"),
			@ApiImplicitParam(name = "storehouseId", value = "仓房ID", paramType = "query"),
			@ApiImplicitParam(name = "roleId", value = "保管员id", paramType = "query")
	})
	public PageInfo<BasicKeeper> getKeeperList(Integer pageNum, Integer pageSize, 
			String keeperName, Integer storehouseId, Integer orgId ,Integer roleId) {
		PageInfo<BasicKeeper> pageInfo = null;
		try {
			pageInfo = keeperService.getKeeperList(pageNum, pageSize, 
					keeperName, storehouseId, orgId ,roleId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return pageInfo;
	}

	@RequestMapping(value="/updateKeeperHouse", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="更新保管员管理的仓房", notes = "更新保管员管理的仓房")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "houseId", value = "仓房ID", paramType = "form"),
			@ApiImplicitParam(name = "receiveId", value = "接收人ID", paramType = "form"),
			@ApiImplicitParam(name = "transferId", value = "移交人ID", paramType = "form"),
			@ApiImplicitParam(name = "roleId", value = "保管员ID", paramType = "form")
	})
	public Map<String, Object> updateKeeperHouse(String houseId, Integer receiveId,Integer transferId,Integer roleId){
		Map<String, Object> modelMap = new HashMap<String,Object>();
		try {
			modelMap = keeperHouseService.updateKeeperHouse(houseId, receiveId, transferId, roleId);
		}catch (Exception ex){
			modelMap.put("msg", "error");
			ex.printStackTrace();
		}
		return modelMap;
	}
}