package com.chinaitop.depot.basic.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.model.BasicStorehouse;
import com.chinaitop.depot.basic.service.BasicEnumService;
import com.chinaitop.depot.basic.service.BasicKeeperHouseService;
import com.chinaitop.depot.basic.service.BasicKeeperService;
import com.chinaitop.depot.basic.service.BasicStorehouseService;
import com.chinaitop.depot.basic.service.BasicTankService;
import com.chinaitop.depot.basic.service.BasicWarehouseService;
import com.chinaitop.depot.common.service.ImportServiceImpl;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 仓房管理Controller，仓房管理菜单中所有的功能都在这里
 *
 * @author User
 *
 */
@RestController
@RequestMapping(value = "/Storehouse")
@Api(value= "BasicStorehouseController", description = "仓房管理控制类")
public class BasicStorehouseController {

	@Resource
	private BasicStorehouseService basicStorehouseService;
	@Resource
	private BasicWarehouseService basicWarehouseService;
	@Resource
	private BasicEnumService enumService;
	@Resource
	private BasicKeeperHouseService keeperHouseService;
	@Resource
	private BasicKeeperService keeperService;
	@Resource
	private ImportServiceImpl importService;
	@Resource
	private BasicTankService basicTankService;


	@Value("${web.upload-path}")
	private String uploadPath;

	@RequestMapping(value = "/getList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="查询仓房信息数据列表", notes = "查询仓房信息数据列表，支持分页")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
			@ApiImplicitParam(name = "storeName", value = "仓房名称", paramType = "query"),
			@ApiImplicitParam(name = "houseId", value = "仓房ID", paramType = "query"),
			@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query"),
			@ApiImplicitParam(name = "storehouseState", value = "仓房状态", paramType = "query"),
			@ApiImplicitParam(name = "storehouseType", value = "仓房类型", paramType = "query"),
			@ApiImplicitParam(name = "libraryType", value = "粮库类型", paramType = "query"),
			@ApiImplicitParam(name = "delFlag", value = "是否删除", paramType = "query")
	})
	public PageInfo<BasicStorehouse> getList(Integer pageNum, Integer pageSize, Integer storehouseType,
											 String storeName, Integer houseId, Integer orgId, Integer storehouseState, 
											 String libraryType, Integer delFlag) {
		//参数对象
		Map<String, Object> modelMap = new HashMap<String, Object>();

		PageInfo<BasicStorehouse> pageInfo = null;
		try {
			if (null != storehouseType) {
				modelMap.put("storehouseType",storehouseType);
			}
			if (null != storeName && !"".equals(storeName)) {
				modelMap.put("storehouseName",storeName);
			}
			if (null != houseId) {
				modelMap.put("storehouseId",houseId);
			}
			if (null != storehouseState) {
				modelMap.put("storehouseState",storehouseState);
			}
			if (null != libraryType) {
				modelMap.put("libraryType",libraryType);
			}

			/* 只能查询当前单位的数据  */
			if (null != orgId) {
				modelMap.put("orgId",orgId);
			}

			// 值为1是没删除的数据(在用),0是已删除的数据(弃用)
			if (null != delFlag) {
				modelMap.put("delFlag",delFlag);
			}
			
			pageInfo = basicStorehouseService.findBasicStorehouseNew(modelMap, orgId, pageNum, pageSize, libraryType);;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return pageInfo;
	}

	@RequestMapping(value = "/findByStorehouse", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="根据ID查找仓房信息(包括保管员)", notes = "根据ID查找仓房信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "id", value = "数据ID", paramType = "query"),
			@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query")
	})
	public BasicStorehouse findByStorehouse(Integer id, Integer orgId) {
		BasicStorehouse basicStorehouse = null;
		try {
			if (id != null) {
				basicStorehouse = basicStorehouseService.findByStorehouse(id, orgId);
			} else {
				basicStorehouse = new BasicStorehouse();
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
		return basicStorehouse;
	}

	@RequestMapping(value = "/getStorehouseList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="根据单位查找单位下的所有仓房信息(统一对外接口)", notes = "根据单位查找单位下的所有仓房信息(统一对外接口)")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "unitId", value = "单位ID", paramType = "query"),
			@ApiImplicitParam(name = "libraryType", value = "仓房性质(主粮库或代储点)", paramType = "query")
	})
	public Map<String, Object> getStorehouseList(Integer unitId, String libraryType) throws Exception {
		Map<String, Object> map = null;
		map = basicStorehouseService.getStorehouseData(unitId, libraryType);
		return map;
	}

	@RequestMapping(value = "/edit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
	@ApiOperation(value="更新仓房信息", notes = "更新仓房信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "storehouseJson", value = "仓房JSON数据对象", paramType = "form"),
			@ApiImplicitParam(name = "warehouseJson", value = "货位数据", paramType = "form"),
			@ApiImplicitParam(name = "userInfo", value = "JSON登录对象", paramType = "form")
	})
	public Map<String, Object> edit(HttpServletRequest request, String storehouseJson, String warehouseJson, String userInfo) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		try {
			modelMap = basicStorehouseService.updateOrSave(request, storehouseJson, warehouseJson, userInfo);
		} catch (Exception e) {
			e.printStackTrace();
			modelMap.put("status", "error");
			modelMap.put("msg", "保存失败！");
		}
		return modelMap;
	}

	@RequestMapping(value = "/remove", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
	@ApiOperation(value="按仓房ID逻辑删除仓房信息", notes = "按仓房ID逻辑删除仓房信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "id", value = "数据ID", paramType = "form")
	})
	public Map<String, Object> remove(Integer id) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		try {
			modelMap = basicStorehouseService.removeStorehouse(id);
		} catch (Exception e) {
			e.printStackTrace();
			modelMap.put("status", "error");
			modelMap.put("msg", "操作失败！");
		}
		return modelMap;
	}

	@RequestMapping(value = "/findStoreHouseListByRole", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="根据登录人角色不同显示不同仓房列表", notes = "根据登录人角色不同显示不同仓房列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
			@ApiImplicitParam(name = "houseNo", value = "仓房ID", paramType = "query"),
			@ApiImplicitParam(name = "userInfo", value = "登录人数据对象JSON", paramType = "query"),
			@ApiImplicitParam(name = "userRoleList", value = "登录人角色数据JSON", paramType = "query")
	})
	public PageInfo<BasicStorehouse> findStoreHouseList(Integer pageNum, Integer pageSize, Integer houseNo, String userInfo, String userRoleList) {
		
		PageInfo<BasicStorehouse> pageInfo = null;

		try {
			pageInfo = basicStorehouseService.findStoreHousePageList(pageNum, pageSize, houseNo, userInfo, userRoleList);
		} catch (Exception e) {
			e.printStackTrace();
		}

		return pageInfo;
	}

	@RequestMapping(value = "/findStoreHouseListByLoginUser", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="查询当前登录人用户的角色为保管员 所管理的仓房", notes = "查询当前登录人用户的角色为保管员 所管理的仓房")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userInfo", value = "登录人数据对象JSON", paramType = "query")
	})
	public Map<String, Object> findStoreHouseListByLoginUser(String userInfo) {
		Map<String, Object> paramMap = new HashMap<String, Object>();
		try {
			paramMap = basicStorehouseService.findStorehouseListByLoginUser(userInfo);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return paramMap;
	}

	@RequestMapping(value="/importFile", method=RequestMethod.POST)
	@ApiOperation(value="导入仓房货位excel", notes = "导入仓房货位excel")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "file", value = "文件数据", paramType = "form"),
			@ApiImplicitParam(name = "fileType", value = "文件名称", paramType = "form"),
			@ApiImplicitParam(name = "userInfo", value = "JSON登录对象", paramType = "form"),
	})
	public String importFile(MultipartFile file, String fileType, String userInfo, HttpServletRequest request){

		JSONObject user = null;
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
		}

		String[] titleArr = {"仓房编号", "仓房名称", "仓房类型", "设计仓容(吨)", "实际仓容(吨)","通风方式", "保管方式", "装粮线高度(m)",
				 "仓房状态","交付使用日期", "货位编号", "货位名称", "货位容量(吨)","粮库类型(代储点是1,正式粮库是0)","代储点粮库名称"};//第一行的title
		boolean[] emptyArr = {false, false, false, true, false, false, false, true, false, true, true, true, true, false, true};//指定列可为空，不可为空值为false.

		List<List<String>> list = new ArrayList<List<String>>();//该list为数据返回的list
		String errormsg = null;
		try {
			errormsg = importService.importFile(uploadPath, file, fileType, request, titleArr, emptyArr, list);

			if (errormsg != null && !"".equals(errormsg)) {
				return errormsg;
			}

			if (list.size()>0) {
				errormsg = basicStorehouseService.importData(list, user);

				System.out.println(list);
				System.out.println(errormsg);
				//errormsg = "导入成功！";
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return errormsg;
	}

	@RequestMapping(value="/findStoreCode", method=RequestMethod.GET)
	@ApiOperation(value="防止仓房编码重复添加", notes = "storehouseCode参数")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "storehouseCode", value = "仓房编码", paramType = "query"),
		@ApiImplicitParam(name = "storeId", value = "仓房ID", paramType = "query"),
		@ApiImplicitParam(name = "libraryType", value = "判断是代储点还是主库", paramType = "query")
		})
	public String findStoreCode(HttpServletRequest request, String storehouseCode, Integer storeId,String libraryType) {
		String codeFlag = "";
		try {
			codeFlag = basicStorehouseService.findStoreCode(request, storehouseCode, storeId, libraryType);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return codeFlag;
	}

	@RequestMapping(value = "/getAgentList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="查询代储库仓房", notes = "查询代储库下边的仓房信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "depotId", value = "代储库ID", paramType = "query"),
			@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query"),
			@ApiImplicitParam(name = "libraryType", value = "粮库类型", paramType = "query"),
	})
	public List<BasicStorehouse> getAgentList(Integer depotId,Integer orgId,String libraryType) {
		List<BasicStorehouse> list = null;
		try {
			list = basicStorehouseService.getAgentList(depotId, orgId, libraryType);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}

	@RequestMapping(value = "/getThreeTempCheckList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="根据单位查找单位下的所有仓房信息(统一对外接口)", notes = "根据单位查找单位下的所有仓房信息(统一对外接口)")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgId", value = "粮库ID", paramType = "query")
	})
	public Map<String, Object> getThreeTempCheckList(Integer orgId) throws Exception {
		Map<String, Object> map = null;
		map = basicStorehouseService.getThreeTempCheckList(orgId);
		return map;
	}

	/**
	 * 根据仓房ID获取仓房name
	 */
	@RequestMapping(value="/getStoreDataByHouseId",produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="根据仓房ID获取仓房name", notes = "根据仓房ID获取仓房name")
	public Map<String, Object> getStoreDataByHouseId(int houseId) {
		Map<String, Object> map = new HashMap<String, Object>();
		BasicStorehouse basicStorehouse = basicStorehouseService.getStoreDataByHouseId(houseId);
		String  StorehouseName = "";
		if(basicStorehouse!=null){
			if(basicStorehouse.getDepotName()!=null){
				StorehouseName = basicStorehouse.getStorehouseName();
			}
		}
		map.put("StorehouseName", StorehouseName);
		return map;
	}
}