package com.chinaitop.depot.basic.controller;

import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import com.chinaitop.depot.basic.model.BasicWarehouseVO;
import org.apache.commons.lang.ObjectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.basic.model.BasicWarehouse;
import com.chinaitop.depot.basic.model.BasicWarehouseExample;
import com.chinaitop.depot.basic.service.BasicWarehouseService;
import com.chinaitop.utils.ParameterUtil;
import com.chinaitop.utils.WarehouseDataSynchronization;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 货位管理Controller，货位管理菜单中所有的功能都在这里
 * @author User
 *
 */
@RestController
@RequestMapping(value="/Warehouse")
@Api(value= "BasicWarehouseController", description = "货位管理控制类")
public class BasicWarehouseController { 

	final static Logger logger = LoggerFactory.getLogger(BasicWarehouse.class);

	@Resource
	private BasicWarehouseService basicWarehouseService;

	@RequestMapping(value="/getList", method = RequestMethod.GET)
	@ApiOperation(value="查询 货位 信息列表", notes = "查询 货位 信息列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "storehouseId", value = "仓房ID", paramType = "query"),
		@ApiImplicitParam(name = "warehouseId", value = "货位ID", paramType = "query"),
		@ApiImplicitParam(name = "warehouseName", value = "货位名称", paramType = "query"),
		@ApiImplicitParam(name = "graindepotId", value = "单位ID", paramType = "query"),
		@ApiImplicitParam(name = "libraryType", value = "粮库类型", paramType = "query"),
		@ApiImplicitParam(name = "delFlag", value = "是否删除", paramType = "query")
	})
	public PageInfo<BasicWarehouse> getList(Integer pageNum, Integer pageSize, Integer storehouseId, Integer warehouseId, 
			String warehouseName, Integer graindepotId, String libraryType, Integer delFlag) {
		PageInfo<BasicWarehouse> pageInfo = null;
		try {
			pageInfo = basicWarehouseService.getPageList(pageNum, pageSize, storehouseId, warehouseId, 
					warehouseName, graindepotId, libraryType, delFlag);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return pageInfo;
	}

	@RequestMapping(value="/findByWarehouse", method = RequestMethod.GET)
	@ApiOperation(value="按货位ID查询一条数据", notes = "按货位ID查询一条数据")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "数据ID", paramType = "query")
	})
	public BasicWarehouse findByWarehouse(Integer id) {
		BasicWarehouse basicWarehouse = new BasicWarehouse();
		if (id != null) {
			basicWarehouse = basicWarehouseService.findByParmaryKey(id);
		}
		return basicWarehouse;
	}

	@RequestMapping(value="/edit", method = RequestMethod.POST)
	@ApiOperation(value="更新货位信息", notes = "更新货位信息")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "warehouseJson", value = "JSON货位数据对象", paramType = "query"),
		@ApiImplicitParam(name = "userInfo", value = "JSON登录数据对象", paramType = "query")
	})
	public Map<String, Object> edit(String warehouseJson, String userInfo) {
		Map<String, Object> map = new HashMap<String, Object>();
        try {
			map = basicWarehouseService.updateOrSave(warehouseJson, userInfo);
		} catch (Exception e) {
			e.printStackTrace();
			map.put("status", "error");
            map.put("msg", "保存失败！");
		}

		return map;
	}

	@RequestMapping(value="/remove", method = RequestMethod.POST)
	@ApiOperation(value="删除货位信息", notes = "删除货位信息")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "数据ID", paramType = "form")
	})
	public Map<String, Object> remove(Integer id, Integer storehouseId) {
		Map<String, Object> map = new HashMap<String, Object>();
        try {
			map = basicWarehouseService.qyYNoN(id, storehouseId);
		} catch (Exception e) {
			e.printStackTrace();
			map.put("status", "error");
            map.put("msg", "操作失败！");
		}
		
		return map;
	}

	@RequestMapping(value="/getWarehouse", method = RequestMethod.GET)
	@ApiOperation(value="根据单位和仓房查询货位信息（货位对外接口）", notes = "根据单位和仓房查询货位信息（货位对外接口）")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "unitId", value = "单位ID", paramType = "query"),
		@ApiImplicitParam(name = "storehouseId", value = "仓房ID", paramType = "query"),
		@ApiImplicitParam(name = "libraryType", value = "库类型(0主粮库,1代储库)", paramType = "query")
	})
	public Map<String, Object> getWarehouse(Integer unitId, Integer storehouseId, String libraryType) {
		Map<String, Object> map = null;
		
		map = basicWarehouseService.getWarehouse(unitId, storehouseId, libraryType);
		
		return map;
	}
	
	@RequestMapping(value="/findWareCode", method=RequestMethod.GET)
	@ApiOperation(value="防止货位编码重复添加", notes = "storehouseCode参数")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "storehouseId", value = "仓房ID", paramType = "query"),
			@ApiImplicitParam(name = "warehouseCode", value = "货位编码", paramType = "query"),
			@ApiImplicitParam(name = "wareId", value = "货位ID", paramType = "query"),
			@ApiImplicitParam(name = "libraryType", value = "判断是代储点还是主库", paramType = "query")
			})
	public String findWareCode(HttpServletRequest request, Integer storehouseId, String warehouseCode, Integer wareId, String libraryType) {
		String codeFlag = "";
		try {
			codeFlag = basicWarehouseService.findWareCode(request, storehouseId, warehouseCode, wareId, libraryType);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return codeFlag;
	}
	
	//库点信息同步
	@SuppressWarnings("all")
	@RequestMapping("/warehouseSynchronization")
	@ApiOperation(value="货位信息同步", notes = "货位信息同步")
	public void orgInfoSynchronization() {
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
		List list = WarehouseDataSynchronization.queryWarehouse();
		BasicWarehouseExample wareExample = new BasicWarehouseExample();
		List<BasicWarehouse> wareList = basicWarehouseService.findCondition(wareExample);
		for(BasicWarehouse warehouse : wareList) {
			int num = 0;
			for(int i=0;i<list.size();i++) {
				Map<String,Object> map = (Map<String,Object>)list.get(i);
				String hwbh = ObjectUtils.toString(map.get("hwbh"),"");//库点编码
				String refreshTime = ObjectUtils.toString(map.get("refreshTime"),"");
				if(hwbh.equals(ObjectUtils.toString(warehouse.getWarehouseCode(),""))) {
					if(!refreshTime.equals(format.format(warehouse.getUpdatedate()))) {
						WarehouseDataSynchronization.warehouseSynchronizationUpdate(map, warehouse);
					}
					break;
				}else if(num == (list.size()-1)){
					WarehouseDataSynchronization.warehouseSynchronizationInsert(warehouse);
				}else {
					num = num+1;
				}
			}
		}
	}

	@RequestMapping(value="/updateWarehouseCrkStatus", method=RequestMethod.POST)
	@ApiOperation(value="修改货位的出入库状态值", notes = "修改货位的出入库状态值")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "wareId", value = "货位ID", paramType = "form"),
		@ApiImplicitParam(name = "crk_status", value = "出入库状态(1空仓,2入库中,3封仓,4出库中,9其它)", paramType = "form")
	})
	public void updateWarehouseCrkStatus(String wareId, String crk_status) {

		String date = ParameterUtil.getSysDateTime();
		logger.info("时间："+date+"，出入库状态回写接口，封仓调用，接收参数是，货位ID："+wareId+"，状态值："+crk_status);
		try {
			if (null != wareId && null != crk_status && !"".equals(crk_status)) {
				basicWarehouseService.updateWarehouseCrkStatus(wareId, crk_status);
				logger.info("时间："+date+"，出入库状态回写接口，封仓调用，调用回写成功！");
			} else {
				logger.info("时间："+date+"，封仓时更新出入库状态值crk_status出错，参数：货位ID="+wareId+"，出入库状态值="+crk_status);
			}
		} catch (Exception e) {
			logger.info("时间："+date+"，出入库状态回写接口，封仓调用，调用失败，报错如下！");
			e.printStackTrace();
		}
	}


	/**
	 * 根据货位编码编码获取有关信息用于和市平台对接
	 */
	@RequestMapping(value="/getWareDataByHwbm",produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="根据货位编码获取有关信息用于和市平台对接", notes = "根据货位编码获取有关信息用于和市平台对接")
	public Map<String, Object> getWareDataByHwbm(String hwbm) {
		Map<String, Object> map = new HashMap<String, Object>();
		BasicWarehouseVO basicWarehouse = basicWarehouseService.getWareDataByHwbm(hwbm);
		Integer houseId = 0;
		String storehouseName = "";
		Integer wareHouseId = 0;
		String warehouseName = "";
		Integer orgId = 0;
		String  transUnitName = "";
		if(basicWarehouse!=null){
			if(basicWarehouse.getStorehouseId()!=null){
				houseId = basicWarehouse.getStorehouseId();
			}
			if(basicWarehouse.getWarehouseId()!=null){
				wareHouseId = basicWarehouse.getWarehouseId();
			}
			if(basicWarehouse.getWarehouseName()!=null){
				warehouseName = basicWarehouse.getWarehouseName();
			}
			if(basicWarehouse.getOrgId()!=null){
				orgId =  basicWarehouse.getOrgId();
			}
			if(basicWarehouse.getDepotName()!=null){
				transUnitName = basicWarehouse.getDepotName();
			}
			if(basicWarehouse.getStorehouseName()!=null){
				storehouseName = basicWarehouse.getStorehouseName();
			}
		}
		map.put("houseId", houseId);
		map.put("wareHouseId", wareHouseId);
		map.put("warehouseName", warehouseName);
		map.put("orgId", orgId);
		map.put("transUnitName", transUnitName);
		map.put("storehouseName", storehouseName);
		return map;
	}
}
