package com.chinaitop.depot.basic.service;

import com.chinaitop.depot.basic.model.BasicKeeper;
import com.chinaitop.depot.basic.model.BasicKeeperExample;
import com.github.pagehelper.PageInfo;

import java.util.List;
import java.util.Map;

public interface BasicKeeperService {

	/**
	 * 添加
	 * @param keeper
	 */
	public void add(BasicKeeper keeper);
	
	/**
	 * 删除
	 * @param id
	 */
	public void delete(Integer id);
	
	/**
	 * 修改
	 * @param keeper
	 */
	public void update(BasicKeeper keeper);
	
	/**
	 * 查询一条数据
	 * @param id
	 * @return
	 */
	public BasicKeeper findByIdParmartKey(Integer id);

	/**
	 * 按照ID查询一个保管员和他的所有学历等信息
	 * @param id
	 * @return
	 */
	public Map<String, Object> findByKeeperAll(Integer id);

	/**
	 * 查询一组数据
	 * @param example
	 * @return
	 */
	public List<BasicKeeper> findCondition(BasicKeeperExample example);

	/**
	 * 获取当前保管员所在粮库的所有仓房和当前保管员的权限
	 * @param unitId 当前保管员所在单位
	 * @param id 当前保管员的ID
	 * @param libraryType 库类型，0.主粮库，1.代储点粮库
	 * @return
	 */
	public Map<String, Object> getUnitStoreKeepList(Integer unitId, Integer id, String libraryType);
	/**
	 * 保管员仓房权限提交
	 * @param selected 被选中的数据
	 * @param userInfo 登录数据对象
	 * @param bgyId 保管员ID
	 * @return
	 */
	public void saveKeeperHouse(String selected, Integer bgyId, String userInfo);

	/**
	 * 更新保管员基本信息和各种证书信息
	 * @param xwxlJson
	 * @param zyjsJson
	 * @param zyzgJson
	 * @param sgzJson
	 * @throws Exception 
	 */
	public void editKeeper(String userInfo, String keeper, String xwxlJson, String zyjsJson, String zyzgJson, String sgzJson) throws Exception;

	/**
	 * 根据移交人的id更新保管员的仓号
	 * @param houseId
	 * @param transferId
	 */
	public void removeHouseKeeperByTransferId(Integer houseId,Integer transferId);

	/**
	 * 根据接收人的id更新保管员的仓号
	 * @param houseId
	 * @param receiveId
	 */
	public void addHouseKeeperbyReceiveId(Integer houseId,Integer receiveId);

	/**
	 * 根据用户的id得到除当前登录人之外的其他保管员的id
	 * 
	 * @param userId 用户ID
	 * @return
	 */
	public BasicKeeper findKeeperHouseByUserId(Integer userId,Integer roleId);

	/**
	 * 根据仓房id获取到仓管员信息
	 * 
	 * @param houseId
	 * @return
	 */
	List<BasicKeeper> getKepperByHouseId(String houseId);

	/**
	 * 根据用户获取保管员管理的全部仓房
	 * 
	 * @param userInfo
	 * @return
	 * @throws Exception 
	 */
	List<BasicKeeper> findBarn(String userInfo) throws Exception;

	/**
	 * 根据名字和仓房id
	 *@param storehouseId
	 * @param name
	 * @return
	 */
	List<BasicKeeper> getKeeperListByNameAndHouseId(String name,Integer storehouseId);

	/**
	 * 获取一批员工的员工名称
	 * 
	 * @param keep_list
	 * @return
	 */
	public String getKeeperNames(List<Integer> keep_list);

	/**
	 * 查询员工数据列表，支持分页
	 * 
	 * @param pageNum
	 * @param pageSize
	 * @param keeperName
	 * @param orgId
	 * @param userId
	 * @return
	 * @throws Exception 
	 */
	public PageInfo<BasicKeeper> getPageList(Integer pageNum, Integer pageSize, String keeperName, Integer orgId,
			Integer userId) throws Exception;

	/**
	 * 通过单位ID，获取保管员数据组(保管员数据对外接口)
	 * 
	 * @param orgId
	 * @return
	 * @throws Exception 
	 */
	public List<BasicKeeper> getKeeperList(Integer orgId) throws Exception;

	/**
	 * 更新员工信息
	 * 
	 * @param keeper
	 * @param xwxlJson
	 * @param zyjsJson
	 * @param zyzgJson
	 * @param sgzJson
	 * @param userInfo
	 * @return
	 * @throws Exception
	 */
	public Map<String, Object> updateOrSave(String keeper, String xwxlJson, String zyjsJson, String zyzgJson,
			String sgzJson, String userInfo) throws Exception;

	/**
	 * 逻辑删除员工信息
	 * 
	 * @param id
	 * @param orgId
	 * @return
	 * @throws Exception
	 */
	public Map<String, Object> removeKeep(Integer id, Integer orgId) throws Exception;

	/**
	 * 查询当前保管员所在库中的所有保管员（排除掉当前登录的保管员用户）
	 * 
	 * @param userInfo
	 * @param roleId
	 * @return
	 * @throws Exception 
	 */
	public List<BasicKeeper> qtKeeper(String userInfo, Integer roleId) throws Exception;

	/**
	 * 查询当前登录用户所对应的保管员信息
	 * 
	 * @param userId
	 * @param roleId
	 * @return
	 * @throws Exception 
	 */
	public BasicKeeper getKeeperByLoginUser(Integer userId, Integer roleId) throws Exception;

	/**
	 * 查询当前仓所对应的保管员
	 * 
	 * @param houseId
	 * @return
	 * @throws Exception 
	 */
	public BasicKeeper getKeeperNamesByHouseId(String houseId) throws Exception;

	/**
	 * 查询保管员数据列表，支持分页
	 * 
	 * @param pageNum
	 * @param pageSize
	 * @param keeperName
	 * @param storehouseId
	 * @param orgId
	 * @param roleId
	 * @return
	 * @throws Exception 
	 */
	public PageInfo<BasicKeeper> getKeeperList(Integer pageNum, Integer pageSize, String keeperName,
			Integer storehouseId, Integer orgId, Integer roleId) throws Exception;
}
