package com.chinaitop.depot.basic.service;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.model.BasicStorehouse;
import com.chinaitop.depot.basic.model.BasicStorehouseExample;
import com.github.pagehelper.PageInfo;

public interface BasicStorehouseService {

	/**
	 * 新增仓房信息
	 * 
	 * @param basicStorehouse
	 * @throws Exception
	 */
	public void addBasicStorehouse(BasicStorehouse basicStorehouse) throws Exception;

	/**
	 * 删除仓房信息
	 * 
	 * @throws Exception
	 */
	public void removeBasicStorehouse(Integer id) throws Exception;

	/**
	 * 修改仓房信息
	 * 
	 * @throws Exception
	 */
	public void updateBasicStorehouse(BasicStorehouse basicStorehouse) throws Exception;

	/**
	 * 按照主键ID查询一条数据
	 * 
	 * @param id
	 * @return
	 * @throws Exception
	 */
	public BasicStorehouse findByPrimary(Integer id) throws Exception;

	/**
	 * 按条件查询一组数据
	 * 
	 * @param basicStorehouseExample
	 * @return
	 * @throws Exception
	 */
	public List<BasicStorehouse> findBasicStorehouse(BasicStorehouseExample basicStorehouseExample) throws Exception;

	/**
	 * 按条件查询一组数据
	 *
	 * @param map
	 * @return
	 * @throws Exception
	 */
	public List<BasicStorehouse> findBasicStorehouseNew(Map<String, Object> map) throws Exception;
	
	/**
	 * 按条件查询一组数据，支持分页
	 *
	 * @param map
	 * @param orgId
	 * @param pageNum
	 * @param pageSize
	 * @param libraryType
	 * @return PageInfo
	 * @throws Exception
	 */
	public PageInfo<BasicStorehouse> findBasicStorehouseNew(Map<String, Object> map, Integer orgId, Integer pageNum, Integer pageSize, String libraryType) throws Exception;

	/**
	 * 查询符合条件的数据总数
	 * 
	 * @param basicStorehouseExample
	 * @return
	 * @throws Exception
	 */
	public int count(BasicStorehouseExample basicStorehouseExample) throws Exception;

	/**
	 * 按照单位ID和粮库类型获取仓房数据（统一对外接口）
	 * 
	 * @param depotId 单位ID
	 * @param libraryType 库类型，0.主粮库，1.代储点粮库
	 * @return
	 */
	public Map<String, Object> getStorehouseData(Integer depotId, String libraryType);

	/**
	 * 导入
	 * 
	 * @param list
	 * @param depotInfo 
	 * @return
	 */
	public String importData(List<List<String>> list, JSONObject userInfo);

	/**
	 * 根据仓房ID获取当前仓房的保管员名称
	 * 
	 * @param id 仓房ID
	 * @param orgId 单位ID
	 * @param flag 是否需要格式化省略
	 * @return
	 */
	public String getStorehouseKeeper(Integer id, Integer orgId, boolean flag);
	
	/**
	 * 查询三温检查列表最新数据所对应的仓房
	 * 
	 * @param orgId 粮库ID
	 * @return
	 */
	public Map<String, Object> getThreeTempCheckList(Integer orgId);



	public BasicStorehouse getStoreDataByHouseId(int houseId);

	/**
	 * 获取一个仓房信息和他的保管员
	 * 
	 * @param id
	 * @param orgId
	 * @return
	 * @throws Exception 
	 */
	public BasicStorehouse findByStorehouse(Integer id, Integer orgId) throws Exception;

	/**
	 * 更新仓房信息
	 * @param storehouseJson
	 * @param warehouseJson
	 * @param userInfo 
	 * @return
	 * @throws Exception 
	 */
	public Map<String, Object> updateOrSave(HttpServletRequest request, String storehouseJson, String warehouseJson, String userInfo) throws Exception;

	/**
	 * 按仓房ID逻辑删除仓房信息
	 * 
	 * @param id
	 * @return
	 * @throws Exception
	 */
	public Map<String, Object> removeStorehouse(Integer id) throws Exception;

	/**
	 * 根据登录人角色不同显示不同仓房列表
	 * 
	 * @param pageNum
	 * @param pageSize
	 * @param houseNo 
	 * @param userInfo
	 * @param userRoleList
	 * @return
	 * @throws Exception
	 */
	public PageInfo<BasicStorehouse> findStoreHousePageList(Integer pageNum, Integer pageSize, Integer houseNo, String userInfo,
			String userRoleList) throws Exception;

	/**
	 * 查询当前登录人用户的角色为保管员 所管理的仓房
	 * 
	 * @param userInfo
	 * @return
	 * @throws Exception
	 */
	public Map<String, Object> findStorehouseListByLoginUser(String userInfo) throws Exception;

	/**
	 * 仓房编码校验
	 * 
	 * @param request 
	 * @param storehouseCode
	 * @param storeId
	 * @param libraryType
	 * @return
	 * @throws Exception
	 */
	public String findStoreCode(HttpServletRequest request, String storehouseCode, Integer storeId, String libraryType) throws Exception;

	/**
	 * 查询代储仓
	 * 
	 * @param depotId
	 * @param orgId
	 * @param libraryType
	 * @return
	 * @throws Exception
	 */
	public List<BasicStorehouse> getAgentList(Integer depotId, Integer orgId, String libraryType) throws Exception;

	/**
	 * 更新单位表仓房数、油罐数、有效仓容、有效罐容
	 * 
	 * @param orgId
	 * @throws Exception
	 */
	void editOrgChNum(Integer orgId) throws Exception;
}
