package com.chinaitop.depot.basic.service;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import com.chinaitop.depot.basic.model.BasicStorehouse;
import com.chinaitop.depot.basic.model.BasicWarehouse;
import com.chinaitop.depot.basic.model.BasicWarehouseExample;
import com.chinaitop.depot.basic.model.BasicWarehouseVO;
import com.github.pagehelper.PageInfo;

public interface BasicWarehouseService {

	/**
	 * 新增
	 * @param basicWarehouse
	 */
	void add(BasicWarehouse basicWarehouse);
	
	/**
	 * 修改
	 * @param basicWarehouse
	 */
	void update(BasicWarehouse basicWarehouse);
	
	/**
	 * 删除
	 * @param id
	 */
	void delete(Integer id);
	
	/**
	 * 按ID查询一条数据
	 * @param id
	 * @return
	 */
	BasicWarehouse findByParmaryKey(Integer id);
	
	/**
	 * 按条件查询一组数据
	 * @param example
	 * @return
	 */
	List<BasicWarehouse> findCondition(BasicWarehouseExample example);
	
	/**
	 * 货位信息对外接口
	 * @param unitId 单位ID
	 * @param storehouseId 仓房ID
	 * @param libraryType库类型(0主粮库,1代储库)
	 * @return
	 */
	Map<String, Object> getWarehouse(Integer unitId, Integer storehouseId, String libraryType);

	/**
	 * 按仓房ID删除货位数据
	 * 
	 * @param storehouseId 仓房ID
	 */
	void deletEexample(Integer storehouseId);

	/**
	 * 查询货位列表，按仓房名称排序
	 * 
	 * @param storehouseId
	 * @param warehouseId
	 * @param warehouseName
	 * @param graindepotId
	 * @param libraryType
	 * @param delFlag
	 * @return
	 */
	List<BasicWarehouse> getList(Integer storehouseId, Integer warehouseId, String warehouseName, Integer graindepotId,
			String libraryType, Integer delFlag);

	/**
	 * 查询货位分页列表
	 * 
	 * @param pageNum
	 * @param pageSize
	 * @param storehouseId
	 * @param warehouseId
	 * @param warehouseName
	 * @param graindepotId
	 * @param libraryType
	 * @param delFlag
	 * @return
	 */
	PageInfo<BasicWarehouse> getPageList(Integer pageNum, Integer pageSize, Integer storehouseId, Integer warehouseId,
			String warehouseName, Integer graindepotId, String libraryType, Integer delFlag) throws Exception;

	/**
	 * 更新货位信息
	 * 
	 * @param warehouseJson
	 * @param userInfo
	 * @return
	 * @throws Exception
	 */
	Map<String, Object> updateOrSave(String warehouseJson, String userInfo) throws Exception;

	/**
	 * 
	 * @param id
	 * @param storehouseId
	 * @return
	 * @throws Exception
	 */
	Map<String, Object> qyYNoN(Integer id, Integer storehouseId) throws Exception;

	/**
	 * 校验货位编码重复
	 * 
	 * @param request
	 * @param storehouseId
	 * @param warehouseCode
	 * @param wareId
	 * @param libraryType
	 * @return
	 * @throws Exception
	 */
	String findWareCode(HttpServletRequest request, Integer storehouseId, String warehouseCode, Integer wareId,
			String libraryType) throws Exception;

	/**
	 * 修改货位的出入库状态值
	 * @param wareId
	 * @param crk_status
	 * @throws Exception
	 */
	void updateWarehouseCrkStatus(String wareId, String crk_status) throws Exception;

	/**
	 * 更新货位数据
	 * 
	 * @param basicStorehouse
	 * @param warehouseJson
	 * @throws Exception
	 */
	void saveOrUpdate(BasicStorehouse basicStorehouse, String warehouseJson) throws Exception;

	BasicWarehouseVO getWareDataByHwbm(String hwbm);
}
