/**
 * 
 */
package com.chinaitop.depot.basic.service.impl;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.mapper.BasicCodeRuleDetailMapper;
import com.chinaitop.depot.basic.mapper.BasicCodeRuleMapper;
import com.chinaitop.depot.basic.model.BasicCodeRule;
import com.chinaitop.depot.basic.model.BasicCodeRuleDetail;
import com.chinaitop.depot.basic.model.BasicCodeRuleDetailExample;
import com.chinaitop.depot.basic.model.BasicCodeRuleDetailExample.Criteria;
import com.chinaitop.depot.basic.model.BasicCodeRuleExample;
import com.chinaitop.depot.basic.model.BasicCodeRuleRecord;
import com.chinaitop.depot.basic.model.BasicCodeRuleRecordExample;
import com.chinaitop.depot.basic.model.Enum.CodeRuleDetailType;
import com.chinaitop.depot.basic.model.Enum.CodeRuleType;
import com.chinaitop.depot.basic.service.BasicCodeRuleDetailService;
import com.chinaitop.depot.basic.service.BasicCodeRuleRecordService;
import com.chinaitop.depot.basic.service.BasicCodeRuleService;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

/**
 * <p>Title: BasicCodeRuleServiceImpl</p>
 * <p>Description: </p>
 * @author yaoyabin
 * @date 2017年12月15日 上午11:00:30
 */
@Service
@SuppressWarnings("all")
public class BasicCodeRuleServiceImpl implements BasicCodeRuleService {
    @Autowired
    private BasicCodeRuleMapper basicCodeRuleMapper;
    @Autowired
    private BasicCodeRuleDetailMapper basicCodeRuleDetailMapper;
    @Autowired
    private BasicCodeRuleDetailService basicCodeRuleDetailService;
    @Autowired
    private BasicCodeRuleRecordService basicCodeRuleRecordService;

    @Override
    public List<BasicCodeRule> queryByExample(Integer pageNum,
            Integer pageSize, Integer orgId) {
        BasicCodeRuleExample example = new BasicCodeRuleExample();
        example.createCriteria().andOrgIdEqualTo(orgId);
        if (null != pageNum && null != pageSize) {
        	PageHelper.startPage(pageNum, pageSize);
        }
        List<BasicCodeRule> list = basicCodeRuleMapper.selectByExample(example);
        return list;
    }

    @Override
    public BasicCodeRule findById(Integer id) {
        return basicCodeRuleMapper.selectByPrimaryKey(id);
    }

    @Override
    public Map<String, Object> add(BasicCodeRule codeRule,
            List<BasicCodeRuleDetail> codeRuleDetailList, Integer userId,
            Integer depotId) {
    	if (null != userId && !"".equals(userId)) {
    		codeRule.setCreater(userId);
    	}
        codeRule.setCreateTime(new Date());
        codeRule.setOrgId(depotId);
        basicCodeRuleMapper.insert(codeRule);
        
        if (codeRuleDetailList != null && codeRuleDetailList.size() > 0) {
            for (BasicCodeRuleDetail basicCodeRuleDetail : codeRuleDetailList) {
                basicCodeRuleDetail.setZid(codeRule.getId());
                basicCodeRuleDetail.setOrgId(depotId+"");
                basicCodeRuleDetailService.add(basicCodeRuleDetail);
            }
        }
        return null;
    }

    @Override
    public void update(BasicCodeRule codeRule,
            List<BasicCodeRuleDetail> codeRuleDetailList) {
        codeRule.setUpdateTime(new Date());
        basicCodeRuleMapper.updateByPrimaryKey(codeRule);
        
        if (codeRuleDetailList != null && codeRuleDetailList.size() > 0) {
            for (BasicCodeRuleDetail basicCodeRuleDetail : codeRuleDetailList) {
                basicCodeRuleDetail.setZid(codeRule.getId());
                basicCodeRuleDetail.setOrgId(codeRule.getOrgId()+"");
                if (basicCodeRuleDetail.getId() == null) {
                    basicCodeRuleDetailService.add(basicCodeRuleDetail);
                } else {
                    basicCodeRuleDetailService.update(basicCodeRuleDetail);
                }
            }
        }
    }

    @Override
    public void remove(Integer id) {
        basicCodeRuleMapper.deleteByPrimaryKey(id);
    }

    @Override
    public List<BasicCodeRuleDetail> getDetailList(Integer pageNum,
            Integer pageSize, Integer bid) {
        BasicCodeRuleDetailExample example = new BasicCodeRuleDetailExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andZidEqualTo(bid);
        example.setOrderByClause("id");
        List<BasicCodeRuleDetail> list = basicCodeRuleDetailService.queryByExample(example);
        return list;
    }

    @Override
    public List<BasicCodeRuleDetail> getDetailList(Integer id) {
        BasicCodeRuleDetailExample example = new BasicCodeRuleDetailExample();
        Criteria createCriteria = example.createCriteria();
        createCriteria.andZidEqualTo(id);
        example.setOrderByClause("id");
        List<BasicCodeRuleDetail> list = basicCodeRuleDetailService.queryByExample(example);
        return list;
    }

    // 通过枚举获取编码.
    @Override
    public String getCode(Integer codeRuleTypeEnumId, Integer orgId) throws Exception {
        StringBuffer sb = new StringBuffer();
        // 通过编码类型的枚举id,获取对应的变量类型.
        BasicCodeRuleExample example = new BasicCodeRuleExample();
        example.setOrderByClause("id desc");
        com.chinaitop.depot.basic.model.BasicCodeRuleExample.Criteria criteria = example.createCriteria();
        criteria.andCodeTypeEqualTo(codeRuleTypeEnumId);
        criteria.andOrgIdEqualTo(orgId);
        List<BasicCodeRule> codeRuleList = basicCodeRuleMapper.selectByExample(example);
        if (codeRuleList != null && codeRuleList.size() > 0) {
            BasicCodeRule basicCodeRule = codeRuleList.get(0);
            
            // 分隔符.
            String spacer = basicCodeRule.getSpacer();
            // 流水长度.
            Integer serialNumberLength = basicCodeRule.getSerialNumberLength();
            
            // 获取编码规则明细.
            BasicCodeRuleDetailExample detailExample = new BasicCodeRuleDetailExample();
            Criteria createCriteria = detailExample.createCriteria();
            createCriteria.andZidEqualTo(basicCodeRule.getId());
            
            List<BasicCodeRuleDetail> detailList = basicCodeRuleDetailService.queryByExample(detailExample);
            
            
            
            if (detailList != null && detailList.size() > 0) {
                for (BasicCodeRuleDetail basicCodeRuleDetail : detailList) {
                    if (CodeRuleDetailType.orgCodeType.getValue().equals(basicCodeRuleDetail.getRuleType())) {
                        // 组织编码.
                        if (StringUtils.isNotBlank(basicCodeRuleDetail.getRuleValue())) {
                            sb.append(basicCodeRuleDetail.getRuleValue());
                            // 拼接分隔符.
                            if (StringUtils.isNoneBlank(spacer)) {
                                sb.append(spacer);
                            }
                        }
                    } else if (CodeRuleDetailType.customType.getValue().equals(basicCodeRuleDetail.getRuleType())) {
                        // 自定义.
                        if (StringUtils.isNotBlank(basicCodeRuleDetail.getRuleValue())) {
                            sb.append(basicCodeRuleDetail.getRuleValue());
                            // 拼接分隔符.
                            if (StringUtils.isNoneBlank(spacer)) {
                                sb.append(spacer);
                            }
                        }
                    } else if (CodeRuleDetailType.dateType.getValue().equals(basicCodeRuleDetail.getRuleType())) {
                        // 日期.
                        if (StringUtils.isNotBlank(basicCodeRuleDetail.getRuleValue())) {
                            SimpleDateFormat sdf = new SimpleDateFormat(basicCodeRuleDetail.getRuleValue());
                            String formatDate = sdf.format(new Date());
                            sb.append(formatDate);
                            // 拼接分隔符.
                            if (StringUtils.isNoneBlank(spacer)) {
                                sb.append(spacer);
                            }
                        }
                    }
                }
            }
            String codeTemplate = sb.toString();
            // 流水.
            BasicCodeRuleRecordExample recordExample = new BasicCodeRuleRecordExample();
            com.chinaitop.depot.basic.model.BasicCodeRuleRecordExample.Criteria recordCriteria = recordExample.createCriteria();
            // 编码类型.
            recordCriteria.andCodeTypeEqualTo(codeRuleTypeEnumId);
            // 编码模板.
            recordCriteria.andCodeTemplateEqualTo(codeTemplate);
            recordExample.setOrderByClause("create_time desc");
            PageHelper.startPage(1, 1);
            List<BasicCodeRuleRecord> recordList = basicCodeRuleRecordService.queryByExample(recordExample);
            
            
            if (recordList != null && recordList.size() > 0) {
                BasicCodeRuleRecord basicCodeRuleRecord = recordList.get(0);
                // 最新一条数据的编码值.
                String codeValue = basicCodeRuleRecord.getCodeValue();
                if (StringUtils.isNotBlank(codeValue)) {
                    // 已经存在此编码的值.
                    String numberS = codeValue.substring(codeTemplate.length(), codeValue.length());
                    int numberI = Integer.parseInt(numberS);
                    numberI = numberI + 1;
                    numberS = numberI + "";
                    if (numberS.length() > serialNumberLength) {
                        throw new RuntimeException("编号长度已经超过流水号最大长度！请重新设定流水号长度！");
                    }
                    String codeNumberNow = getCodeNumber(serialNumberLength, numberI);
                    sb.append(codeNumberNow);
                } else {
                    // 第一次.
                    String codeNumberNow = getCodeNumber(serialNumberLength, 1);
                    sb.append(codeNumberNow);
                }
            } else {
                // 第一次.
                String codeNumberNow = getCodeNumber(serialNumberLength, 1);
                sb.append(codeNumberNow);
            }
            
            // 将生成的编码，插入到编码记录表.
            BasicCodeRuleRecord record = new BasicCodeRuleRecord();
            record.setCodeType(codeRuleTypeEnumId);
            record.setCodeTemplate(codeTemplate);
            record.setCodeValue(sb.toString());
            record.setCreateTime(new Date());
            record.setOrgId(orgId);
            basicCodeRuleRecordService.add(record);
        } else {
            throw new RuntimeException("未定义该类型的编码规则！请先定义编码规则！");
        }
        return sb.toString();
    }
    
    private String getCodeNumber(Integer length, Integer num) {
        return String.format("%0"+length+"d", num);
    }

	@Override
	public List<BasicCodeRule> findCodeName(BasicCodeRuleExample example) {
		return basicCodeRuleMapper.selectByExample(example);
	}

	@Override
	public PageInfo<BasicCodeRule> getPageList(Integer pageNum, Integer pageSize, Integer orgId) throws Exception {
		List<BasicCodeRule> list = this.queryByExample(pageNum, pageSize, orgId);
		PageInfo<BasicCodeRule> pageInfo = new PageInfo<BasicCodeRule>(list);
		return pageInfo;
	}

	@Override
	public PageInfo<BasicCodeRuleDetail> getPageListBasicCodeRuleDetail(Integer pageNum, Integer pageSize, Integer bid)
			throws Exception {
		List<BasicCodeRuleDetail> list = this.getDetailList(pageNum, pageSize, bid);
		PageInfo<BasicCodeRuleDetail> pageInfo = new PageInfo<BasicCodeRuleDetail>(list);
		return pageInfo;
	}

	@Override
	public Map<String, Object> findByIdConditionMap(Integer id) throws Exception {

		Map<String, Object> returnMap = new HashMap<String, Object>();

		// 主表数据.
		BasicCodeRule codeRule = this.findById(id);
		// 子表数据.
		List<BasicCodeRuleDetail> codeRuleDetailList = this.getDetailList(id);

		returnMap.put("codeRule", codeRule);
		returnMap.put("codeRuleDetailList", codeRuleDetailList);

		return returnMap;
	}

	@Override
	public Map<String, Object> updateOrSave(String codeRuleJson, String codeRuleDetailJson, String userInfo) throws Exception {

		//返回结果对象
		Map<String, Object> modelMap = new HashMap<String, Object>();

		//解析查询条件
		JSONObject user = null;
		Integer orgId = null;
		Integer userId = null;
		BasicCodeRuleExample example = new BasicCodeRuleExample();
		BasicCodeRuleExample.Criteria criteria = example.createCriteria();
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
			orgId = Integer.parseInt(user.get("orgId").toString());
			userId = Integer.parseInt(user.get("userId").toString());
		}

		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		BasicCodeRule codeRule = (BasicCodeRule) mapper.readValue(codeRuleJson, BasicCodeRule.class);
		List<BasicCodeRuleDetail> codeRuleDetailList = null;
		if (codeRuleDetailJson != null) {
			codeRuleDetailList = (List<BasicCodeRuleDetail>) mapper.readValue(codeRuleDetailJson,
					new TypeReference<List<BasicCodeRuleDetail>>() {
					});
		}
		criteria.andCodeTypeEqualTo(codeRule.getCodeType());
		criteria.andOrgIdEqualTo(orgId);
		List<BasicCodeRule> list = this.findCodeName(example);
		//如果ID存在则表明是修改数据
		if (codeRule.getId() == null) {
			//再次判断是否存在该编码规则，不存在则新增，否则弹框提示
			if (list.size() == 0 && list.isEmpty()) {
				this.add(codeRule, codeRuleDetailList, userId, orgId);
				modelMap.put("status", "success");
			} else {
				modelMap.put("status", "error");
				modelMap.put("msg", "该编码名称已存在,请勿重复添加！");
				return modelMap;
			}
		} else {
			this.update(codeRule, codeRuleDetailList);
			modelMap.put("status", "success");
		}
		return modelMap;
	}

	@Override
	public Map<String, Object> removeCodeRuleAll(Integer id) throws Exception {

		Map<String, Object> modelMap = new HashMap<String, Object>();
		// 删除主表数据
		this.remove(id);
		// 删除关联子表数据
		BasicCodeRuleDetailExample example = new BasicCodeRuleDetailExample();
		BasicCodeRuleDetailExample.Criteria criteria = example.createCriteria();
		criteria.andZidEqualTo(id);
		basicCodeRuleDetailMapper.deleteByExample(example);
		
		modelMap.put("status", "success");
		return null;
	}

	@Override
	public Map<String, Object> getCodeValue(String codeRuleType, Integer orgId) throws Exception {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		try {
			if (orgId == null) {
				throw new RuntimeException("参数orgId为空");
			}
		} catch (Exception e) {
			modelMap.put("status", "error");
			modelMap.put("msg", "未获取到对应的组织机构，无法获取到编码！");
			return modelMap;
		}

		if (StringUtils.isNotBlank(codeRuleType)) {
			Integer codeRuleTypeEnumId = CodeRuleType.getEnumIdByType(codeRuleType);
			if (codeRuleTypeEnumId != null) {
				String codeValue = "";
				try {
					codeValue = this.getCode(codeRuleTypeEnumId, orgId);
				} catch (Exception e) {
					e.printStackTrace();
					modelMap.put("status", "error");
					modelMap.put("msg", e.getMessage());
					return modelMap;
				}
				modelMap.put("status", "success");
				modelMap.put("codeValue", codeValue);
			} else {
				modelMap.put("status", "error");
				modelMap.put("msg", "编码类型为空！");
			}
		} else {
			modelMap.put("status", "error");
			modelMap.put("msg", "编码类型为空！");
		}
		return modelMap;
	}

}
