package com.chinaitop.depot.basic.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;

import com.chinaitop.depot.basic.mapper.BasicKeeperHouseMapper;
import com.chinaitop.depot.basic.model.BasicKeeper;
import com.chinaitop.depot.basic.model.BasicKeeperHouse;
import com.chinaitop.depot.basic.model.BasicKeeperHouseExample;
import com.chinaitop.depot.basic.service.BasicKeeperHouseService;
import com.chinaitop.depot.basic.service.BasicKeeperService;

@Service
public class BasicKeeperHouseServiceImpl implements BasicKeeperHouseService {

	@Resource
	private BasicKeeperHouseMapper mapper;

	@Resource
	private BasicKeeperService keeperService;

	@Override
	public void save(BasicKeeperHouse keeperHouse) {
		mapper.insertSelective(keeperHouse);
	}

	@Override
	public void delete(BasicKeeperHouseExample example) {
		mapper.deleteByExample(example);
	}

	@Override
	public List<BasicKeeperHouse> findCondition(BasicKeeperHouseExample example) {
		return mapper.selectByExample(example);
	}

	@Override
	public void deleteByKeeperIdAndHouseId(Integer keeperId, String houseId) {
		mapper.deleteByKeeperIdAndHouseId(keeperId,houseId);
	}

	@Override
	public BasicKeeperHouse getHouseKeeperBykeeperIdAndHouseId(Integer keeperId, String houseId) {
		return  mapper.getHouseKeeperBykeeperIdAndHouseId(keeperId,houseId);
	}

	@Override
	public List<Integer> getKeeperId(Integer orgId, String houseId) {
		BasicKeeperHouseExample keepHouseExample = new BasicKeeperHouseExample();
		BasicKeeperHouseExample.Criteria criteria = keepHouseExample.createCriteria();
		criteria.andHouseIdEqualTo(houseId);
		criteria.andOrgIdEqualTo(orgId);
		List<BasicKeeperHouse> keepHouse_list = mapper.selectByExample(keepHouseExample);

		/* 获取保管这个仓房的多有人的员工ID */
		List<Integer> keep_list = null;
		if (null != keepHouse_list && keepHouse_list.size() > 0) {
			keep_list = new ArrayList<Integer>();
			for (BasicKeeperHouse keeperHouse : keepHouse_list) {
				keep_list.add(keeperHouse.getKeeperId());
			}
		}
		return keep_list;
	}

	@Override
	public Map<String, Object> updateKeeperHouse(String houseId, Integer receiveId, Integer transferId,
			Integer roleId) {
		Map<String, Object> modelMap = new HashMap<String,Object>();

		/**保管员移交步骤
		 * 更新移交人和接收人在仓房和保管员表中中的记录basic_keeper_house表
		 */
		//更新移交人和接收人在仓房和保管员表中的记录basic_keeper_house表
		//得到对应的保管员
		BasicKeeper keeper = keeperService.findKeeperHouseByUserId(transferId, roleId);
		//根据移交人的id和仓号删除在basic_keeper_house表中的记录
		mapper.deleteByKeeperIdAndHouseId(keeper.getId(), houseId);

		//获取接收人的id
		Integer rId = keeperService.findKeeperHouseByUserId(receiveId,roleId).getId();
		BasicKeeperHouse keeperHouse = new BasicKeeperHouse();
		keeperHouse.setHouseId(houseId + "");//移交的仓房
		keeperHouse.setKeeperId(rId);//接收人的id
		keeperHouse.setOrgId(keeper.getGrainId());//登录的人所在库点
		keeperHouse.setCreateId(keeper.getId());//登录人（移交人）
		keeperHouse.setCreateTime(new Date());

		//如果移交人和接收人管理的有相同的仓号，
		// 那么移交后会出现house_keeper_houer表的keeperId和houseId相同，就会出现报错，这里特殊处理
		BasicKeeperHouse basicKeeperHouse = mapper.getHouseKeeperBykeeperIdAndHouseId(rId,houseId);
		if(basicKeeperHouse == null){
			//进行保存
			mapper.insertSelective(keeperHouse);
		}
		modelMap.put("msg","success");

		return modelMap;
	}

}
