package com.chinaitop.depot.basic.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.ObjectUtils;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.mapper.BasicEnumMapper;
import com.chinaitop.depot.basic.mapper.BasicKeeperEducationMapper;
import com.chinaitop.depot.basic.mapper.BasicKeeperHouseMapper;
import com.chinaitop.depot.basic.mapper.BasicKeeperMapper;
import com.chinaitop.depot.basic.mapper.BasicStorehouseMapper;
import com.chinaitop.depot.basic.mapper.BasicTankMapper;
import com.chinaitop.depot.basic.model.BasicHouseTank;
import com.chinaitop.depot.basic.model.BasicKeeper;
import com.chinaitop.depot.basic.model.BasicKeeperEducation;
import com.chinaitop.depot.basic.model.BasicKeeperEducationExample;
import com.chinaitop.depot.basic.model.BasicKeeperExample;
import com.chinaitop.depot.basic.model.BasicKeeperHouse;
import com.chinaitop.depot.basic.model.BasicKeeperHouseExample;
import com.chinaitop.depot.basic.model.BasicStorehouse;
import com.chinaitop.depot.basic.model.BasicStorehouseExample;
import com.chinaitop.depot.basic.model.BasicStorehouseExample.Criteria;
import com.chinaitop.depot.basic.model.BasicTank;
import com.chinaitop.depot.basic.model.BasicTankExample;
import com.chinaitop.depot.basic.service.BasicKeeperService;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

@Service
@SuppressWarnings("all")
public class BasicKeeperServiceImpl implements BasicKeeperService {

	@Resource
	private BasicKeeperMapper mapper;

	@Resource
	private BasicStorehouseMapper storehouseMapper;

	@Resource
	private BasicKeeperHouseMapper keepHouseMapper;

	@Resource
	private BasicKeeperEducationMapper eduMapper;

	@Resource
	private BasicEnumMapper enumMapper;

	@Resource
	private BasicTankMapper tankMapper;
	
	private final static String KEEPER_NAME = "('保管员')";

	@Override
	public void add(BasicKeeper keeper) {
		mapper.insertSelective(keeper);
	}

	@Override
	public void delete(Integer id) {
		mapper.deleteByPrimaryKey(id);
	}

	@Override
	public void update(BasicKeeper keeper) {
		mapper.updateByPrimaryKeySelective(keeper);
	}

	@Override
	public BasicKeeper findByIdParmartKey(Integer id) {
		return mapper.selectByPrimaryKey(id);
	}

	@Override
	public List<BasicKeeper> findCondition(BasicKeeperExample example) {
		return mapper.selectByExample(example);
	}

	@Override
	public Map<String, Object> findByKeeperAll(Integer id) {
		Map<String, Object> map = new HashMap<String, Object>();
		BasicKeeper keeper = null;

		List<BasicKeeperEducation> xlxwList = new ArrayList<BasicKeeperEducation>(); //学历学位信息1
		List<BasicKeeperEducation> zyjsList = new ArrayList<BasicKeeperEducation>(); //专业技术职称2
		List<BasicKeeperEducation> zyzgList = new ArrayList<BasicKeeperEducation>(); //职业资格及等级3
		List<BasicKeeperEducation> sgzList = new ArrayList<BasicKeeperEducation>(); //上岗证4
		BasicKeeperEducation ekeObje = null;

		if (id != null) {
			//获取保管员基础信息
			keeper = mapper.selectByPrimaryKey(id);

			//获取这个保管员可管理的仓房
			BasicKeeperHouseExample storeKeepExample = new BasicKeeperHouseExample();
			BasicKeeperHouseExample.Criteria keepCriteria = storeKeepExample.createCriteria();
			keepCriteria.andKeeperIdEqualTo(id);
			storeKeepExample.setOrderByClause("house_Id asc");
			List<BasicKeeperHouse> kepHouselist = keepHouseMapper.selectByExample(storeKeepExample);

			// 将仓房ID插入一个Integer类型的集合，将油罐ID插入一个String类型的集合，用于转换名称时的条件
			List<Integer> idList = new ArrayList<Integer>();
			List<String> strList = new ArrayList<String>();
			if (null != kepHouselist && kepHouselist.size() > 0) {
				for (BasicKeeperHouse bkh : kepHouselist) {
					String hid = bkh.getHouseId();
					//ID要是字符串，那就是油罐，负责就是粮食仓房
					if (hid.length() > 2 && "YG".equals(hid.substring(0, 2).toString())) {
						strList.add(hid);
					} else {
						idList.add(Integer.parseInt(hid));
					}
				}
			}

			// 将粮食仓房ID转换为仓房名称
			if (idList.size() > 0) {
				BasicStorehouseExample storeExample = new BasicStorehouseExample();
				Criteria criteria = storeExample.createCriteria();
				criteria.andOrgIdEqualTo(keeper.getGrainId());
				criteria.andStorehouseIdIn(idList);
				criteria.andDelFlagEqualTo(1);
				List<BasicStorehouse> storelist = storehouseMapper.selectByExample(storeExample);
				if (null != storelist && storelist.size() > 0) {
					String storeName = "";
					for (BasicStorehouse bsh : storelist) {
						storeName += bsh.getStorehouseName() + ",";
					}
					storeName = storeName.substring(0, storeName.length()-1);
					keeper.setHouseId(storeName);
				}
			}

			// 将油罐ID转换为油罐名称
			if (strList.size() > 0) {
				BasicTankExample tankExample = new BasicTankExample();
				BasicTankExample.Criteria criteria1 = tankExample.createCriteria();
				criteria1.andOrgIdEqualTo(keeper.getOrgId());
				criteria1.andIdIn(strList);
				criteria1.andDelFlagEqualTo(1);
				List<BasicTank> tanklist = tankMapper.selectByExample(tankExample);
				if (null != tanklist && tanklist.size() > 0) {
					String tankName = "";
					for (BasicTank tank : tanklist) {
						tankName += tank.getStoragetankName() + ",";
					}
					tankName = tankName.substring(0, tankName.length()-1);
					keeper.setHouseId(keeper.getHouseId() + "," + tankName);
				}
			}

			//获取保管员学历、职业等其它数据集合
			BasicKeeperEducationExample example = new BasicKeeperEducationExample();
			BasicKeeperEducationExample.Criteria educationCriteria = example.createCriteria();
			educationCriteria.andKeeperIdEqualTo(id);
			example.setOrderByClause("id,dataType asc");
			List<BasicKeeperEducation> list = eduMapper.selectByExample(example);

			if (null != list && list.size() > 0) {
				for (BasicKeeperEducation bke : list) {
					bke.setKeeperId(keeper.getId());
					if (1 == bke.getDatatype()) {
						bke.setDatatype(1);
						xlxwList.add(bke);
					} else if (2 == bke.getDatatype()) {
						bke.setDatatype(2);
						zyjsList.add(bke);
					} else if (3 == bke.getDatatype()) {
						bke.setDatatype(3);
						zyzgList.add(bke);
					} else if (4 == bke.getDatatype()) {
						bke.setDatatype(4);
						sgzList.add(bke);
					}
				}
			}
		} else {
			keeper = new BasicKeeper();
		}



		map.put("keeper", keeper);


		//如果有一种类型的集合是空的，那么先给new一个空对象
		if (null == xlxwList || xlxwList.size() == 0) {
			ekeObje = new BasicKeeperEducation();
			ekeObje.setKeeperId(keeper.getId());
			ekeObje.setDatatype(1);
			xlxwList.add(ekeObje);
		}
		if (null == zyjsList || zyjsList.size() == 0) {
			ekeObje = new BasicKeeperEducation();
			ekeObje.setKeeperId(keeper.getId());
			ekeObje.setDatatype(2);
			zyjsList.add(ekeObje);
		}
		if (null == zyzgList || zyzgList.size() == 0) {
			ekeObje = new BasicKeeperEducation();
			ekeObje.setKeeperId(keeper.getId());
			ekeObje.setDatatype(3);
			zyzgList.add(ekeObje);
		}
		if (null == sgzList || sgzList.size() == 0) {
			ekeObje = new BasicKeeperEducation();
			ekeObje.setKeeperId(keeper.getId());
			ekeObje.setDatatype(4);
			sgzList.add(ekeObje);
		}

		map.put("xlxwList", xlxwList);
		map.put("zyjsList", zyjsList);
		map.put("zyzgList", zyzgList);
		map.put("sgzList", sgzList);

		return map;
	}

	@Override
	public Map<String, Object> getUnitStoreKeepList(Integer orgId, Integer id, String libraryType) {
		Map<String, Object> map = new HashMap<String, Object>();

		// 获取当前保管员所在单位下的所有仓房
		BasicStorehouseExample storeExample = new BasicStorehouseExample();
		Criteria criteria = storeExample.createCriteria();
		criteria.andOrgIdEqualTo(orgId);
		criteria.andDelFlagEqualTo(1);
		if(libraryType!="null" && libraryType!="") {
			criteria.andLibraryTypeEqualTo(libraryType);
		}
		List<BasicStorehouse> storelist = storehouseMapper.selectByExample(storeExample);

		// 获取当前保管员所在单位下的所有油罐
		BasicTankExample tankExample = new BasicTankExample();
		BasicTankExample.Criteria criteria1 = tankExample.createCriteria();
		criteria1.andOrgIdEqualTo(orgId);
		criteria1.andDelFlagEqualTo(1);
		List<BasicTank> tanklist = tankMapper.selectByExample(tankExample);

		// 把油罐和仓房放在一个集合中
		List<BasicHouseTank> house_tank_list = new ArrayList<BasicHouseTank>();
		BasicHouseTank house_tank = null;
		for (BasicStorehouse sto : storelist) {
			house_tank = new BasicHouseTank();
			house_tank.setId(sto.getStorehouseId() + "");
			house_tank.setCode(sto.getStorehouseCode());
			house_tank.setName(sto.getStorehouseName());
			house_tank_list.add(house_tank);
		}
		for (BasicTank tan : tanklist) {
			house_tank = new BasicHouseTank();
			house_tank.setId(tan.getId());
			house_tank.setCode(tan.getStoragetankCode());
			house_tank.setName(tan.getStoragetankName());
			house_tank_list.add(house_tank);
		}

		// 获取当前保管员在他所在单位所拥有的可管理的仓房权限
		BasicKeeperHouseExample storeKeepExample = new BasicKeeperHouseExample();
		BasicKeeperHouseExample.Criteria keepCriteria = storeKeepExample.createCriteria();
		keepCriteria.andOrgIdEqualTo(orgId);
		keepCriteria.andKeeperIdEqualTo(id);
		List<BasicKeeperHouse> kepHouselist = keepHouseMapper.selectByExample(storeKeepExample);

		//设置将已经的有管理权限的的仓房默认选中
		if (null != kepHouselist && kepHouselist.size() > 0) {
			String storeId = null;
			for (BasicHouseTank ht : house_tank_list) {
				storeId = ht.getId();
				for (BasicKeeperHouse kepHouse : kepHouselist) {
					if (kepHouse.getHouseId() == storeId || storeId.equals(kepHouse.getHouseId())) {
						ht.setDelFlag(5); // 利用这个字段做个标识，5代表是这个仓房有管理权限，要默认选中
					}
				}
			}

		}

		map.put("storeList", house_tank_list);

		return map;
	}

	@Override
	public void saveKeeperHouse(String selected, Integer bgyId, String userInfo) {
		JSONObject user = null;
		Integer orgId = null;
		Integer userId = null;
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
			orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
			userId = Integer.parseInt(user.get("userId") == null?"":user.get("userId").toString());
		}
		//先删掉原来的保管员仓房权限数据
		BasicKeeperHouseExample example = new BasicKeeperHouseExample();
		com.chinaitop.depot.basic.model.BasicKeeperHouseExample.Criteria keepCriteria = example.createCriteria();
		keepCriteria.andKeeperIdEqualTo(bgyId);
		keepCriteria.andOrgIdEqualTo(orgId);
		keepHouseMapper.deleteByExample(example);

		String[]checkArray = null;
		if (selected.length() > 2) {
			String sted = selected.substring(1, selected.length() - 1);
			sted = sted.replace("\"", "");
			checkArray = sted.split(",");
		}

		// 在把新的权限信息插入
		BasicKeeperHouse keperHouse = null;
		if (checkArray != null && checkArray.length > 0) {
			Date date = new Date();
			//UserInfo user = (UserInfo) request.getSession().getAttribute("userInfo");
			//for (String hid : checkArray) {
			for (int i = 0; i < checkArray.length; i++) {
				keperHouse = new BasicKeeperHouse();
				keperHouse.setKeeperId(bgyId);
				keperHouse.setOrgId(orgId);
				keperHouse.setHouseId(checkArray[i]);
				keperHouse.setCreateId(userId);
				keperHouse.setCreateTime(date);
				keepHouseMapper.insertSelective(keperHouse);
			}
		}
	}

	@Override
	public void editKeeper(String userInfo, String keeper,
						   String xwxlJson, String zyjsJson, String zyzgJson, String sgzJson) throws Exception {
		Date date = new Date();
		JSONObject user = null;
		Integer orgId = null;
		Integer userId = null;
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
			orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
			userId = Integer.parseInt(user.get("userId") == null?"":user.get("userId").toString());
		}
		// 保存主表数据
		ObjectMapper mappers = new ObjectMapper();
		BasicKeeper basicKeeper = mappers.readValue(keeper, BasicKeeper.class);
		Integer id = null;
		if (basicKeeper.getId() != null) {
			id = basicKeeper.getId();
			basicKeeper.setEditPersonId(userId);
			basicKeeper.setEditDate(date);
			mapper.updateByPrimaryKey(basicKeeper);

			//如果由保管员改变到其它部门后，要把他管理的仓房删掉
			int roleId = basicKeeper.getPostType();

			String roleName = mapper.getRoleName(roleId);
			if (!"保管员".equals(roleName) && !"仓储人员".equals(roleName)) {
				BasicKeeperHouseExample khExample = new BasicKeeperHouseExample();
				BasicKeeperHouseExample.Criteria khCriteria = khExample.createCriteria();
				khCriteria.andKeeperIdEqualTo(id);
				khCriteria.andOrgIdEqualTo(orgId);
				keepHouseMapper.deleteByExample(khExample);
			}
		} else {
			basicKeeper.setGrainId(orgId);
			basicKeeper.setOrgId(orgId);
			basicKeeper.setInputPersonId(userId);
			basicKeeper.setInputDate(date);
			basicKeeper.setDelFlag(1);
			mapper.insertSelective(basicKeeper);
			id = basicKeeper.getId();
		}

		//更新子表数据
		//学历学位信息1
		List<BasicKeeperEducation> xlxwList = mappers.readValue(xwxlJson, new TypeReference<List<BasicKeeperEducation>>(){});
		editEducation(xlxwList, id);
		//专业技术职称2
		List<BasicKeeperEducation> zyjsList = mappers.readValue(zyjsJson, new TypeReference<List<BasicKeeperEducation>>(){});
		editEducation(zyjsList, id);
		//职业资格及等级3
		List<BasicKeeperEducation> zyzgList = mappers.readValue(zyzgJson, new TypeReference<List<BasicKeeperEducation>>(){});
		editEducation(zyzgList, id);
		//上岗证4
		List<BasicKeeperEducation> sgzList = mappers.readValue(sgzJson, new TypeReference<List<BasicKeeperEducation>>(){});
		editEducation(sgzList, id);

	}

	/**
	 * 更新保管员证件信息
	 *
	 * @param edcationList
	 * @param id
	 */
	private void editEducation(List<BasicKeeperEducation> edcationList, Integer id) {
		Integer dataType = 0; //数据类型
		for (BasicKeeperEducation bke : edcationList) {
			if (null == bke.getId() || "".equals(bke.getId())) {
				bke.setKeeperId(id);
				dataType = bke.getDatatype();
				if ("1".equals(dataType) || dataType == 1) {
					Integer type = bke.getType();
					if (type == null) {
						continue;
					}
				} else if ("2".equals(dataType) || dataType == 2) {
					String departMent = bke.getDepartment();
					if (departMent == null || "".equals(departMent)) {
						continue;
					}
				} else if ("3".equals(dataType) || dataType == 3) {
					Integer occupation = bke.getOccupation();
					if (occupation == null) {
						continue;
					}
				}
				eduMapper.insertSelective(bke);
			} else {
				eduMapper.updateByPrimaryKeySelective(bke);
			}
		}
	}

	/**
	 * 根据仓房id获取到仓管员信息
	 * @param houseId
	 * @return
	 */
	@Override
	public List<BasicKeeper> getKepperByHouseId(String houseId) {
		BasicKeeperHouseExample example = new BasicKeeperHouseExample();
		BasicKeeperHouseExample.Criteria criteria = example.createCriteria();
		criteria.andHouseIdEqualTo(houseId + "");
		//example.setDistinct(true);
		List<BasicKeeperHouse> keepHouseList = keepHouseMapper.selectByExample(example);
		List<Integer> list = new ArrayList<Integer>();
		if (null != keepHouseList && keepHouseList.size() > 0) {
			for (BasicKeeperHouse k : keepHouseList) {
				list.add(k.getKeeperId());
			}
		}
		List<BasicKeeper> keeperList = new ArrayList<BasicKeeper>();
		if (list != null && list.size() > 0) {
			BasicKeeperExample keeperExample = new BasicKeeperExample();
			BasicKeeperExample.Criteria kcriteria = keeperExample.createCriteria();
			kcriteria.andIdIn(list);
			keeperList = mapper.selectByExample(keeperExample);
		}
		return keeperList;
	}

	/**
	 * 根据移交人的id更新保管员的仓号 【由于业务变更该方法已经不用  暂时留着】
	 * @param houseId
	 * @param transferId
	 */
	@Override
	public void removeHouseKeeperByTransferId(Integer houseId, Integer transferId) {
		/**
		 * ①根据transferId得到该保管员，并且得到该保管员目前所管理的仓号
		 * ②得到该保管员所管理的仓号和要移交的仓号对比  移除掉要移交的仓号
		 * ③进行更新
		 */

		//①根据transferId得到该保管员
		/*BasicKeeper keeper=this.findKeeperHouseByUserId(transferId);
        String houseIds=keeper.getHouseId();
		String[] houses=houseIds.split(",");
        String houseIId=String.valueOf(houseId);
		//②得到该保管员所管理的仓号和要移交的仓号对比  移除掉要移交的仓号
		List<String> list = new ArrayList<String>();
		try {
			for (int i=0;i<houses.length;i++){
				if(!houses[i].equals(houseIId)){
					list.add(houses[i]);
				}
			}
			StringBuffer sb = new StringBuffer();
			for (String s : list){
				sb.append(s).append(",");
			}
			keeper.setHouseId(sb.substring(0,sb.length()-1).toString());

			//③进行更新
			mapper.updateByPrimaryKey(keeper);
		}catch (Exception ex){
			ex.printStackTrace();
		}*/
	}
	//根据移交人id获取对应的保管员信息
	@Override
	public BasicKeeper findKeeperHouseByUserId(Integer userId,Integer roleId) {
		//BasicEnum basicEnum= enumMapper.selectByPrimaryKey(5417);

		BasicKeeperExample keeperExample = new BasicKeeperExample();
		BasicKeeperExample.Criteria criteria = keeperExample.createCriteria();
		criteria.andUserIdEqualTo(userId);
		criteria.andDelFlagEqualTo(1);
		keeperExample.setOrderByClause("id desc");
		if(roleId != null){
			criteria.andPostTypeEqualTo(roleId);
		}
		List<BasicKeeper> keeperList = mapper.selectByExample(keeperExample);
		BasicKeeper keeper = null;
		if (null != keeperList && keeperList.size() > 0) {
			keeper = keeperList.get(0);
		}
		return keeper;
	}

	/**
	 * 根据接收人的id更新保管员的仓号【由于业务变更该方法已经不用  暂时留着】
	 * @param houseId
	 * @param receiveId
	 */
	@Override
	public void addHouseKeeperbyReceiveId(Integer houseId, Integer receiveId) {
		/**
		 * ①查找对应的保管员  和 对应的仓房的id
		 * ②拼接接收到的仓号
		 * ③更新
		 */

		//①查找对应的保管员  和 对应的仓房的id
		/*BasicKeeper keeper=this.findKeeperHouseByUserId(receiveId);
		String houseIds=keeper.getHouseId();
		String[] houses=houseIds.split(",");
		String houseIId=String.valueOf(houseId);
		//②得到该保管员所管理的仓号和要移交的仓号对比  移除掉要移交的仓号
		List<String> list = new ArrayList<String>();
		try {
			for (int i=0;i<houses.length;i++){
				if(!houses[i].equals(houseIId)){
					list.add(houses[i]);
				}
			}
			list.add(houseIId);//将当前的仓号放进去
			StringBuffer sb = new StringBuffer();
			for (String s : list){
				sb.append(s).append(",");
			}
			keeper.setHouseId(sb.substring(0,sb.length()-1).toString());

			//③进行更新
			mapper.updateByPrimaryKey(keeper);
		}catch (Exception ex){
			ex.printStackTrace();
		}*/

	}

	@Override
	public List<BasicKeeper> findBarn(String userInfo) throws Exception {
		JSONObject user = null;
		Integer userId = null;
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
			userId = Integer.parseInt(user.get("userId") == null?"":user.get("userId").toString());
		}
		Map<String,Object> map = new HashMap<>();
		map.put("userId",userId);
		List<BasicKeeper> keeperList = mapper.selectBarn(map);
		return keeperList;
	}

	@Override
	public List<BasicKeeper> getKeeperListByNameAndHouseId(String name, Integer storehouseId) {
		return mapper.getKeeperListByNameAndHouseId(name,storehouseId);
	}

	@Override
	public String getKeeperNames(List<Integer> keep_list) {

		String keeperNames = "";

		BasicKeeperExample keeperExample = new BasicKeeperExample();
		BasicKeeperExample.Criteria criteria = keeperExample.createCriteria();
		criteria.andIdIn(keep_list);
		List<BasicKeeper> list = mapper.selectByExample(keeperExample);

		if (null != list && list.size() > 0) {
			for (BasicKeeper keeper : list) {
				keeperNames = keeperNames + keeper.getName() + ",";
			}

			keeperNames = keeperNames.substring(0, keeperNames.length()-1);

		}

		return keeperNames;
	}

	@Override
	public PageInfo<BasicKeeper> getPageList(Integer pageNum, Integer pageSize, String keeperName, Integer orgId,
			Integer userId) throws Exception {

		BasicKeeperExample keeperExample = new BasicKeeperExample();
		BasicKeeperExample.Criteria criteria = keeperExample.createCriteria();
		// 拼接查询条件
		if (!"".equals(keeperName) && keeperName != null) {
			criteria.andNameLike("%" + keeperName + "%");
		}
		if (null != orgId && !"".equals(orgId)) {
			criteria.andOrgIdEqualTo(orgId);
		}
		if (null != userId && !"".equals(userId)) {
			criteria.andUserIdEqualTo(userId);
		}
		criteria.andDelFlagEqualTo(1); // 查询正常数据
		if (null != pageNum && null != pageSize) {
			PageHelper.startPage(pageNum, pageSize);
		}
		keeperExample.setOrderByClause(" id desc");
		List<BasicKeeper> list = this.findCondition(keeperExample);
		PageInfo<BasicKeeper> pageInfo = new PageInfo<BasicKeeper>(list);

		return pageInfo;
	}

	@Override
	public List<BasicKeeper> getKeeperList(Integer orgId) throws Exception {
		//查询保管员的角色ID
		Map<String, Object> param_map = new HashMap<String, Object>();
		param_map.put("orgId", orgId);
		param_map.put("names", this.KEEPER_NAME);
		String ids = mapper.getKeeperRoleIds(param_map);
		List<Integer> ids_list = null;

		//根据保管员角色ID查询库里的保管员
		List<BasicKeeper> list = null;
		if (null != ids && !"".equals(ids)) {
			String []id_nums = ids.split(",");
			ids_list = new ArrayList<Integer>();
			for (int i = 0; i < id_nums.length; i++) {
				ids_list.add(Integer.parseInt(id_nums[i]));
			}
			BasicKeeperExample keeperExample = new BasicKeeperExample();
			BasicKeeperExample.Criteria criteria = keeperExample.createCriteria();
			criteria.andOrgIdEqualTo(orgId);
			criteria.andPostTypeIn(ids_list);
			criteria.andDelFlagEqualTo(1); // 查询正常数据
			list = this.findCondition(keeperExample);
		}
		return list;
	}

	@Override
	public Map<String, Object> updateOrSave(String keeper, String xwxlJson, String zyjsJson, String zyzgJson,
			String sgzJson, String userInfo) throws Exception {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		ObjectMapper mappers = new ObjectMapper();
		BasicKeeper basicKeeper = mappers.readValue(keeper, BasicKeeper.class);
		if (basicKeeper.getId() == null) {
			//校验重复添加
			BasicKeeperExample keeperExample = new BasicKeeperExample();
			BasicKeeperExample.Criteria criteria = keeperExample.createCriteria();
			criteria.andUserIdEqualTo(basicKeeper.getUserId());
			criteria.andPostTypeEqualTo(basicKeeper.getPostType());
			criteria.andDelFlagEqualTo(1);
			List<BasicKeeper> kList = this.findCondition(keeperExample);
			if (null != kList && kList.size() > 0) {
				modelMap.put("status", "success");
				modelMap.put("msg", "当前职员在当前岗位的信息已经存在，请不要重复添加！");
				return modelMap;
			}
		}
		this.editKeeper(userInfo, keeper, xwxlJson, zyjsJson, zyzgJson, sgzJson);
		modelMap.put("status", "success");
		modelMap.put("msg", "保存成功！");
		return modelMap;
	}

	@Override
	public Map<String, Object> removeKeep(Integer id, Integer orgId) throws Exception {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		BasicKeeper keeper = new BasicKeeper();
		try {
			if (null != id) {
				keeper.setId(id);
				keeper.setDelFlag(0);
				mapper.updateByPrimaryKeySelective(keeper);
				modelMap.put("status", "success");

				/* 清理掉他所管理的仓房 */
				BasicKeeperHouseExample example = new BasicKeeperHouseExample();
				BasicKeeperHouseExample.Criteria khCriteria = example.createCriteria();
				khCriteria.andKeeperIdEqualTo(id);
				khCriteria.andOrgIdEqualTo(orgId);
				keepHouseMapper.deleteByExample(example);
				modelMap.put("status", "success");
				modelMap.put("msg", "删除成功！");
			}
		} catch (Exception e) {
			e.printStackTrace();
			modelMap.put("status", "error");
			modelMap.put("msg", "删除失败！");
		}

		return modelMap;
	}

	@Override
	public List<BasicKeeper> qtKeeper(String userInfo, Integer roleId) throws Exception {
		JSONObject user = null;
		Integer orgId = null;
		Integer userId = null;
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
			orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
			userId = Integer.parseInt(user.get("userId") == null?"":user.get("userId").toString());
		}
		BasicKeeperExample keeperExample = new BasicKeeperExample();
		BasicKeeperExample.Criteria criteria = keeperExample.createCriteria();
		List<BasicKeeper> list = null;
		// 拼接查询条件单位
		if (null != orgId && !"".equals(orgId)) {
			criteria.andGrainIdEqualTo(orgId);
		}
		//排除掉当前登录的保管员用户，只返回该库除登录人的保管员用户
		if(userId !=null && !"".equals(userId)){
			criteria.andUserIdNotEqualTo(userId);
		}
		criteria.andDelFlagEqualTo(1); // 查询正常数据
		criteria.andPostTypeEqualTo(roleId);//查询类型为保管员的记录
		list = mapper.selectByExample(keeperExample);
		return list;
	}

	@Override
	public BasicKeeper getKeeperByLoginUser(Integer userId, Integer roleId) throws Exception {
		BasicKeeper keeper = new BasicKeeper();
		BasicKeeperExample keeperExample = new BasicKeeperExample();
		BasicKeeperExample.Criteria criteria = keeperExample.createCriteria();
		if(userId !=null && !"".equals(userId)){
			criteria.andUserIdEqualTo(userId);
		}
		if(roleId !=null && !"".equals(roleId)){
			//查询角色是保管员的对应的信息
			criteria.andPostTypeEqualTo(roleId);
		}
		keeperExample.setOrderByClause("id desc");
		List<BasicKeeper> keeperList = mapper.selectByExample(keeperExample);
		if (null != keeperList && keeperList.size() > 0) {
			keeper = keeperList.get(0);
		}
		return keeper;
	}

	@Override
	public BasicKeeper getKeeperNamesByHouseId(String houseId) throws Exception {
		BasicKeeperHouseExample keeperHouseExample = new BasicKeeperHouseExample();
		BasicKeeperHouseExample.Criteria keeperHouseCriteria = keeperHouseExample.createCriteria();
		keeperHouseCriteria.andHouseIdEqualTo(houseId + "");
		BasicKeeper keeper = new BasicKeeper();
		StringBuffer sb = new StringBuffer();
		List<BasicKeeperHouse> keeperHouseList = keepHouseMapper.selectByExample(keeperHouseExample);
		if(keeperHouseList.size()>0 && keeperHouseList != null){
			for(BasicKeeperHouse akh : keeperHouseList){
				keeper = mapper.selectByPrimaryKey(Integer.valueOf(akh.getKeeperId()));
				if(keeper != null){
					sb.append(keeper.getName()+",");
				}
			}
		}
		if(sb.length() != 0){
			keeper.setKeeperNames(sb.substring(0,sb.length()-1));
		}
		return keeper;
	}

	@Override
	public PageInfo<BasicKeeper> getKeeperList(Integer pageNum, Integer pageSize, String keeperName,
			Integer storehouseId, Integer orgId, Integer roleId) throws Exception {
		
		PageInfo<BasicKeeper> pageInfo = null;
		
		BasicKeeperExample keeperExample = new BasicKeeperExample();
		BasicKeeperExample.Criteria criteria = keeperExample.createCriteria();
		List<BasicKeeper> list = null;
		try {
			if (storehouseId == null) {
				// 拼接查询条件
				if (!"".equals(keeperName) && keeperName != null) {
					criteria.andNameLike("%" + keeperName + "%");
				}
				if (!"".equals(roleId) && roleId != null){
					criteria.andPostTypeEqualTo(roleId);//查询保管员
				}
				criteria.andDelFlagEqualTo(1); // 查询正常数据
				if(ParameterUtil.isnotnull(orgId)){
					criteria.andOrgIdEqualTo(orgId);//单位
				}
				if (null != pageNum && null != pageSize) {
					PageHelper.startPage(pageNum, pageSize);
				}
				list = mapper.selectByExample(keeperExample);
			}else{
				if (null != pageNum && null != pageSize) {
					PageHelper.startPage(pageNum, pageSize);
				}
				list = mapper.getKeeperListByNameAndHouseId(keeperName, storehouseId);
			}
			pageInfo = new PageInfo<BasicKeeper>(list);
		} catch (Exception e) {
			e.printStackTrace();
		}

		return pageInfo;
	}

}
