package com.chinaitop.depot.basic.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.ObjectUtils;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.mapper.BasicEnumMapper;
import com.chinaitop.depot.basic.mapper.BasicStorehouseMapper;
import com.chinaitop.depot.basic.mapper.BasicTankMapper;
import com.chinaitop.depot.basic.mapper.BasicWarehouseMapper;
import com.chinaitop.depot.basic.model.BasicEnum;
import com.chinaitop.depot.basic.model.BasicEnumExample;
import com.chinaitop.depot.basic.model.BasicHouseTank;
import com.chinaitop.depot.basic.model.BasicKeeper;
import com.chinaitop.depot.basic.model.BasicKeeperExample;
import com.chinaitop.depot.basic.model.BasicKeeperHouse;
import com.chinaitop.depot.basic.model.BasicKeeperHouseExample;
import com.chinaitop.depot.basic.model.BasicStorehouse;
import com.chinaitop.depot.basic.model.BasicStorehouseExample;
import com.chinaitop.depot.basic.model.BasicStorehouseExample.Criteria;
import com.chinaitop.depot.basic.model.BasicTank;
import com.chinaitop.depot.basic.model.BasicTankExample;
import com.chinaitop.depot.basic.model.BasicWarehouse;
import com.chinaitop.depot.basic.model.BasicWarehouseExample;
import com.chinaitop.depot.basic.service.BasicKeeperHouseService;
import com.chinaitop.depot.basic.service.BasicKeeperService;
import com.chinaitop.depot.basic.service.BasicStorehouseService;
import com.chinaitop.depot.basic.service.BasicTankService;
import com.chinaitop.depot.basic.service.BasicWarehouseService;
import com.chinaitop.depot.utils.BusinessUtils;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

@Service
@SuppressWarnings("all")
public class BasicStorehouseServiceImpl implements BasicStorehouseService {

	@Resource
	private BasicStorehouseMapper basic;

	@Resource
	private BasicWarehouseMapper wareMapper;
	
	@Resource
	private BasicWarehouseService warehouseService;

	@Resource
	private BasicEnumMapper enumMapper;

	@Resource
	private BasicKeeperHouseService keeperHouseService;

	@Resource
	private BasicKeeperService keeperService;
	
	@Resource
	private BasicTankService tankService;

	@Resource
	private BasicTankMapper basicTankMapper;

	@Override
	public void addBasicStorehouse(BasicStorehouse basicStorehouse) throws Exception {
		basic.insertSelective(basicStorehouse);
	}

	@Override
	public void removeBasicStorehouse(Integer id) throws Exception {
		basic.deleteByPrimaryKey(id);
	}

	@Override
	public void updateBasicStorehouse(BasicStorehouse basicStorehouse) throws Exception {
		basic.updateByPrimaryKey(basicStorehouse);
	}

	@Override
	public BasicStorehouse findByPrimary(Integer id) throws Exception {
		return basic.selectByPrimaryKey(id);
	}

	@Override
	public List<BasicStorehouse> findBasicStorehouse(BasicStorehouseExample basicStorehouseExample) throws Exception {
		return basic.selectByExample(basicStorehouseExample);
	}

	@Override
	public List<BasicStorehouse> findBasicStorehouseNew(Map<String, Object> map) throws Exception {
		return basic.selectByExampleNew(map);
	}

	@Override
	public int count(BasicStorehouseExample basicStorehouseExample) throws Exception {
		return basic.countByExample(basicStorehouseExample);
	}

	@Override
	public Map<String, Object> getStorehouseData(Integer orgId, String libraryType) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		BasicStorehouseExample example = new BasicStorehouseExample();
		Criteria criteria = example.createCriteria();
		List<BasicStorehouse> houselist = null;
		try {
			example.setOrderByClause("storehouse_id asc");
			if (ParameterUtil.isnotnull(orgId)) {
				criteria.andOrgIdEqualTo(orgId);
			}
			if (ParameterUtil.isnotnull(libraryType)) {
				criteria.andLibraryTypeEqualTo(libraryType);
			}else if(ParameterUtil.isnull(libraryType)){
				criteria.andLibraryTypeEqualTo("0");//如果不传值则默认查询主库数据，不查询代储库数据
			}
			criteria.andDelFlagEqualTo(1); // 查询正常数据
			houselist = basic.selectByExample(example); //获取当前单位下的所有有效数据
		} catch (Exception e) {
			e.printStackTrace();
		}
		modelMap.put("houseList", houselist);
		modelMap.put("houseObj", getStorehouseObj(orgId,libraryType));
		modelMap.put("houseCodeObj", getStorehouseCodeObj(orgId,libraryType));
		
		//特殊需要，把仓房ID也转换成string类型的数据跟油罐的拼凑在一起返回
		//1、查询油罐基础数据
		BasicTankExample tank_example = new BasicTankExample();
		BasicTankExample.Criteria tank_criteria = tank_example.createCriteria();
		if (ParameterUtil.isnotnull(orgId)) {
			tank_criteria.andOrgIdEqualTo(orgId);
		}
		List<BasicTank> tank_list = tankService.findBasicTank(tank_example);
		
		//2、建立一个放仓房和油罐的数据大对象，并且在建立一个存放仓房和油罐数据对象的list
		BasicHouseTank house_tank = null;
		List<BasicHouseTank> house_tank_list = new ArrayList<BasicHouseTank>();

		//3、放入油罐信息
		for (BasicTank tank : tank_list) {
			house_tank = new BasicHouseTank();
			house_tank.setCode(tank.getStoragetankCode());
			house_tank.setId(tank.getId());
			house_tank.setName(tank.getStoragetankName());
			house_tank_list.add(house_tank);
		}

		//4、放入仓房信息
		for (BasicStorehouse storehouse : houselist) {
			house_tank = new BasicHouseTank();
			house_tank.setCode(storehouse.getStorehouseCode());
			house_tank.setId(storehouse.getStorehouseId()+"");
			house_tank.setName(storehouse.getStorehouseName());
			house_tank_list.add(house_tank);
		}
		
		//5、放入要返回的map对象
		modelMap.put("house_and_tank_list", house_tank_list);

		return modelMap;
	}

	/**
	 * 获取当前单位下所有仓房数据(全局接口，不可随便修改)
	 *
	 * @param orgId
	 * @return
	 */
	private Map<Integer, Object> getStorehouseObj(Integer orgId,String libraryType) {
		BasicStorehouseExample example = new BasicStorehouseExample();
		Criteria criteria = example.createCriteria();
		List<BasicStorehouse> houseObj = null;
		try {
			example.setOrderByClause("storehouse_id asc");

			if (ParameterUtil.isnotnull(libraryType)) {
				criteria.andLibraryTypeEqualTo(libraryType);
			}else if(ParameterUtil.isnull(libraryType)){
				criteria.andLibraryTypeEqualTo("0");//如果不传值则默认查询主库数据，不查询代储库数据
			}
			if (ParameterUtil.isnotnull(orgId)) {
				criteria.andOrgIdEqualTo(orgId);
			}
			
			/*criteria.andDelFlagEqualTo(1); // 查询正常数据
*/			houseObj = basic.selectByExample(example); //获取当前单位所有的仓房数据
		} catch (Exception e) {
			e.printStackTrace();
		}
		Map<Integer, Object> objMap = new HashMap<Integer, Object>();
		for (BasicStorehouse obj : houseObj) {
			objMap.put(obj.getStorehouseId(), obj);
		}
		return objMap;
	}

	/**
	 * 获取当前单位下所有仓房数据-仓房编码(全局接口，不可随便修改)
	 *
	 * @param depotId
	 * @return
	 */
	private Map<String, Object> getStorehouseCodeObj(Integer orgId,String libraryType) {
		BasicStorehouseExample example = new BasicStorehouseExample();
		Criteria criteria = example.createCriteria();
		List<BasicStorehouse> houseObj = null;
		try {
			example.setOrderByClause("storehouse_id asc");

			if (ParameterUtil.isnotnull(libraryType)) {
				criteria.andLibraryTypeEqualTo(libraryType);
			}else if(ParameterUtil.isnull(libraryType)){
				criteria.andLibraryTypeEqualTo("0");//如果不传值则默认查询主库数据，不查询代储库数据
			}
			if (ParameterUtil.isnotnull(orgId)) {
				criteria.andOrgIdEqualTo(orgId);
			}
			criteria.andDelFlagEqualTo(1); // 查询正常数据
			houseObj = basic.selectByExample(example); //获取当前单位所有的仓房数据
		} catch (Exception e) {
			e.printStackTrace();
		}
		Map<String, Object> objMap = new HashMap<>();
		for (BasicStorehouse obj : houseObj) {
			objMap.put(obj.getStorehouseCode(), obj);
		}
		return objMap;
	}

	@Override
	public String importData(List<List<String>> list, JSONObject user) {

		Integer orgId = null;
		String orgName = "";
		if (null != user) {
			orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
			orgName = user.get("orgName") == null?"":user.get("orgName").toString();
		}

		String uuid = null;
		Date date = new Date();
		Map<String, Integer> map = new HashMap<String, Integer>();
		BasicStorehouse storehouse = null;
		BasicWarehouse warehouse = null;
		String msg = "";//消息
		boolean value_flag = false;
		int len = 0;//长度
		//根据字典名字查询字典编号
		BasicEnumExample enumExample = new BasicEnumExample();
		BasicEnumExample.Criteria Criteria = enumExample.createCriteria();
		List<BasicEnum> basicEnums;

		for (int i = 0; i < list.size(); i++) {
			List<String> revList = list.get(i);
			uuid = UUID.randomUUID().toString();

			for (int j = 0; j < revList.size(); j++) {
				switch(j) {
					case 0://仓房编号
						if (ParameterUtil.isnotnull(revList.get(j))) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 1 列的值的长度不能超过50！";
								return msg;
							}

							Integer mapKey = map.get(revList.get(j));
							if (mapKey == null) {
								storehouse = new BasicStorehouse();
								map.put(revList.get(j), null);
								storehouse.setStorehouseCode(revList.get(j));
							}
						}
						break;
					case 1://仓房名称
						if (storehouse != null) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 2 列的值的长度不能超过50！";
								return msg;
							}
							storehouse.setStorehouseName(revList.get(j));
						}
						break;
					case 2://仓房类型
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							enumExample.clear();
							Criteria = enumExample.createCriteria();
							Criteria.andEnumnameEqualTo(revList.get(j));
							basicEnums = enumMapper.selectByExample(enumExample);
							if(basicEnums.size() == 0){
								msg = "第 "+(i+1)+" 行第 3 列的值不正确，请查找后在导入！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 11) {
								msg = "第 "+(i+1)+" 行第 3 列的值的长度不能超过11！";
								return msg;
							}
							storehouse.setStorehouseType(basicEnums.get(0).getEnumid());
						}
						break;
					case 3://设计仓容(吨)
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							len = revList.get(j).length();
							Pattern p = Pattern.compile("^(([1-9]{1}\\d*)|([0]{1}))(\\.(\\d){0,2})?$");
							Matcher m = p.matcher(revList.get(j));
							boolean flag = m.matches();
							if (!flag || len > 30) {
								msg = "第"+(i+1)+" 行的第 4 列的值的长度包括小数位不能大于30位，小数位最多2位，";
								return msg;
							}
							BigDecimal bd = new BigDecimal(revList.get(j));
							storehouse.setDesignCapacity(bd);
						}
						break;
					case 4://实际仓容(吨)
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							len = revList.get(j).length();
							Pattern p = Pattern.compile("^(([1-9]{1}\\d*)|([0]{1}))(\\.(\\d){0,2})?$");
							Matcher m = p.matcher(revList.get(j));
							boolean flag = m.matches();
							if (!flag || len > 30) {
								msg = "第"+(i+1)+" 行的第 5 列的值的长度包括小数位不能大于30位，小数位最多2位，";
								return msg;
							}
							BigDecimal bd = new BigDecimal(revList.get(j));
							storehouse.setActualCapacity(bd);
						}
						break;
					case 5://通风方式
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							enumExample.clear();
							Criteria = enumExample.createCriteria();
							Criteria.andEnumnameEqualTo(revList.get(j));
							basicEnums = enumMapper.selectByExample(enumExample);
							if(basicEnums.size() == 0){
								msg = "第 "+(i+1)+" 行第 6 列的值不正确，请查找后在导入！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 100) {
								msg = "第 "+(i+1)+" 行第 6 列的值的长度不能超过100！";
								return msg;
							}
							storehouse.setDrafttype(basicEnums.get(0).getEnumid());
						}
						break;
					case 6://保管方式
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							enumExample.clear();
							Criteria = enumExample.createCriteria();
							Criteria.andEnumnameEqualTo(revList.get(j));
							basicEnums = enumMapper.selectByExample(enumExample);
							if(basicEnums.size() == 0){
								msg = "第 "+(i+1)+" 行第 7 列的值不正确，请查找后在导入！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 100) {
								msg = "第 "+(i+1)+" 行第 7 列的值的长度不能超过100！";
								return msg;
							}
							storehouse.setKeepingWay(basicEnums.get(0).getEnumid());
						}
						break;
					case 7://装粮线高度(m)
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							value_flag = revList.get(j).matches("[0-9]+");
							if (!value_flag) {
								msg = "第 "+(i+1)+" 行的第 8 列的值只能是数字！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 30) {
								msg = "第 "+(i+1)+" 行第 8 列的值的长度不能超过30！";
								return msg;
							}
							storehouse.setGrainHeigth(new BigDecimal(revList.get(j)));
						}
						break;
					/*case 8://保管员 注：liangs 2019/07/08 注掉保管员列
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 9 列的值的长度不能超过50！";
								return msg;
							}
							storehouse.setDutystoreman(revList.get(j));
						}
						break;*/
					case 8://仓房状态
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							enumExample.clear();
							Criteria = enumExample.createCriteria();
							Criteria.andEnumnameEqualTo(revList.get(j));
							basicEnums = enumMapper.selectByExample(enumExample);
							if(basicEnums.size() == 0){
								msg = "第 "+(i+1)+" 行第 9 列的值不正确，请查找后在导入！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 11) {
								msg = "第 "+(i+1)+" 行第 9 列的值的长度不能超过11！";
								return msg;
							}
							storehouse.setStorehouseState(basicEnums.get(0).getEnumid());
						}
						break;
					case 9://交付使用日期
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							if(!ParameterUtil.stringIsTime(revList.get(j))){
								msg = "第 "+(i+1)+" 行第 10 列的值不属于时间，请确认后再提交！";
								return msg;
							}
							storehouse.setUsedate(ParameterUtil.string2datetime(revList.get(j)));
						}
						if (storehouse != null) {
							storehouse.setUuid(uuid);
							storehouse.setDelFlag(1);
							storehouse.setCreatedate(date);
							storehouse.setOrgId(orgId);
						}
						break;
					case 10://货位编号
						if (ParameterUtil.isnotnull(revList.get(j))) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 11 列的值的长度不能超过50！";
								return msg;
							}

							warehouse = new BasicWarehouse();
							warehouse.setWarehouseCode(revList.get(j));
						}
						break;
					case 11://货位名称
						if (ParameterUtil.isnotnull(revList.get(j))) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 12 列的值的长度不能超过50！";
								return msg;
							}
							if (warehouse != null) {
								warehouse.setWarehouseName(revList.get(j));
							}
						}
						break;
					case 12://货位容量(吨)
						if (ParameterUtil.isnotnull(revList.get(j))) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 13 列的值的长度不能超过50！";
								return msg;
							}
							if (warehouse != null) {
								warehouse.setWarehouseCapacity(new BigDecimal(revList.get(j)));
							}
						}
						if (warehouse != null) {
							warehouse.setDelFlag(1);
							warehouse.setCreatedate(date);
							warehouse.setOrgId(orgId);
							warehouse.setWarehouseStatus(2963); //添加时默认货位状态为空仓
						}
						break;
					case 13:
						if (ParameterUtil.isnotnull(revList.get(j))) {
							//插入仓房信息
							if (storehouse != null) {
								storehouse.setLibraryType(revList.get(j));
							}
							if (warehouse != null) {
								warehouse.setLibraryType(revList.get(j));
							}
						}
						break;
					case 14:
						//if (ParameterUtil.isnotnull(revList.get(j))) {
							Map<String,Object> depotMap = new HashMap<String, Object>();

							//插入仓房信息
							if (storehouse != null) {
								//判断是否有代储粮库
								if ("1".equals(storehouse.getLibraryType())) {
									depotMap.put("agentDepotName",revList.get(j));
									depotMap = basic.getAgentDepotId(depotMap);
									if(ParameterUtil.isnull(depotMap)){
										msg = "第 "+(i+1)+" 行第 15 列的值的不正确，请查询后再添加！";
										return msg;
									}
								}

								if(ParameterUtil.isequal("1",storehouse.getLibraryType())){
									storehouse.setDepotId(Integer.parseInt(depotMap.get("id").toString()));
									storehouse.setDepotName(revList.get(j));
									map.put(storehouse.getDepotName(), storehouse.getDepotId());
								}else{
									storehouse.setDepotId(orgId);
									storehouse.setDepotName(orgName);
								}
								//校验仓房编码不可重复
								boolean flag = uniqueParityCheck("house", storehouse.getStorehouseCode(), orgId, null);
								if (!flag) {
									msg = "第 "+(i+1)+" 行的仓房编码已在数据库存在！";
									return msg;
								}
								basic.insert(storehouse);
								map.put(storehouse.getStorehouseCode(), storehouse.getStorehouseId());
							}
							if (warehouse != null) {
								if(ParameterUtil.isequal("1",warehouse.getLibraryType())){
									warehouse.setDepotId(map.get(revList.get(j)));
									warehouse.setDepotName(revList.get(j));
								}else{
									warehouse.setDepotId(orgId);
									warehouse.setDepotName(orgName);
								}
								//校验货位编码不可重复
								boolean flag = uniqueParityCheck("ware", warehouse.getWarehouseCode(), orgId, map.get(revList.get(0)));
								if (!flag) {
									msg = "第 "+(i+1)+" 行的仓房编码已在数据库存在！";
									return msg;
								}
								warehouse.setStorehouseId(map.get(revList.get(0)));
								wareMapper.insert(warehouse);
							}
						//}
						storehouse = null;
						warehouse = null;
						break;
					default :
						break;

				}

			}

		}
		return msg;
	}

	/**
	 * 校验仓房或货位的编码是否存在
	 * 
	 * @param type 标识符
	 * @param code 编码值
	 * @param orgId 单位ID
	 * @param houseId 仓房ID
	 * @return
	 */
	private boolean uniqueParityCheck(String type, String code, Integer orgId, Integer houseId) {
		boolean flag = false;
		
		if ("house".equals(type)) {
			BasicStorehouseExample example = new BasicStorehouseExample();
			Criteria criteria = example.createCriteria();
			criteria.andOrgIdEqualTo(orgId);
			criteria.andStorehouseCodeEqualTo(code);
			criteria.andLibraryTypeEqualTo("0");
			List<BasicStorehouse> list = basic.selectByExample(example);
			if (null == list || list.size() == 0) {
				flag = true;
			}
		}
		if ("ware".equals(type)) {
			BasicWarehouseExample example = new BasicWarehouseExample();
			BasicWarehouseExample.Criteria criteria = example.createCriteria();
			criteria.andOrgIdEqualTo(orgId);
			criteria.andWarehouseCodeEqualTo(code);
			criteria.andStorehouseIdEqualTo(houseId);
			criteria.andLibraryTypeEqualTo("0");
			List<BasicWarehouse> list = wareMapper.selectByExample(example);
			if (null == list || list.size() == 0) {
				flag = true;
			}
		}
		
		return flag;
	}

	@Override
	public String getStorehouseKeeper(Integer id, Integer orgId, boolean flag) {
		String keeperNames = "";

		/* 获取这个仓房的所有保管员的员工ID */
		List<Integer> keep_list = keeperHouseService.getKeeperId(orgId, id.toString());

		//2、然后把这些人的翻译过来，展示出来
		if (null != keep_list && keep_list.size() > 0) {
			keeperNames = keeperService.getKeeperNames(keep_list);
			if (flag) {
				if (keeperNames.length() > 7) {
					keeperNames = keeperNames.substring(0, 7) + "...";
				}
			}
		}

		return keeperNames;
	}
	

	@Override
	public Map<String, Object> getThreeTempCheckList(Integer orgId) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		Map<String, Object> map = new HashMap<String, Object>();
		List<BasicStorehouse> houselist = null;
		try {
			if (ParameterUtil.isnotnull(orgId)) {
				map.put("orgId", orgId);
			}
			map.put("libraryType", 0);//默认查询主库数据，不查询代储库数据
			map.put("delFlag", 1);//查询正常数据
			houselist = basic.getThreeTempCheckList(map); //获取当前单位下的所有有效数据
		} catch (Exception e) {
			e.printStackTrace();
		}
		modelMap.put("houseList", houselist);
		return modelMap;
	}

	@Override
	public BasicStorehouse getStoreDataByHouseId(int houseId) {
		// TODO Auto-generated method stub
		return basic.getStoreDataByHouseId(houseId);
	}

	@Override
	public PageInfo<BasicStorehouse> findBasicStorehouseNew(Map<String, Object> map, Integer orgId, 
			Integer pageNum, Integer pageSize, String libraryType) throws Exception {
		PageInfo<BasicStorehouse> pageInfo = null;
		if (pageNum != null && pageSize != null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<BasicStorehouse> list = basic.selectByExampleNew(map);
		pageInfo = new PageInfo<BasicStorehouse>(list);
		//获取当前仓房的保管员
		if (null != pageInfo.getList() && pageInfo.getList().size() > 0 && ParameterUtil.isequal("0",libraryType)) {
			String keeperNames = "";
			for (BasicStorehouse storehouse : list) {
				keeperNames = getStorehouseKeeper(storehouse.getStorehouseId(), orgId, true);
				storehouse.setDutystoreman(keeperNames);
			}
		}
		return pageInfo;
	}

	@Override
	public BasicStorehouse findByStorehouse(Integer id, Integer orgId) throws Exception {
		BasicStorehouse basicStorehouse = basic.selectByPrimaryKey(id);
		if(ParameterUtil.isequal(basicStorehouse.getLibraryType(),"0")){
			//获取当前仓房的保管员
			String keeperNames = this.getStorehouseKeeper(id, orgId, false);
			basicStorehouse.setDutystoreman(keeperNames);
		}
		return basicStorehouse;
	}

	@Override
	public Map<String, Object> updateOrSave(HttpServletRequest request, String storehouseJson, String warehouseJson, String userInfo) throws Exception {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		JSONObject user = null;
		Integer orgId = null;
		String userName = "";
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
			orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
			userName = user.get("username") == null?"":user.get("username").toString();
		}
		JSONObject org = null;
		//获取单位信息
		String orgInfo = ObjectUtils.toString(request.getSession().getAttribute("orgInfo"),"");
		String dwbm = "";
		if (null != orgInfo && !"".equals(orgInfo)) {
			org = JSONObject.parseObject(orgInfo);
			dwbm = org.get("creditCode") == null?"":org.get("creditCode").toString();
		}
		Date date = new Date();

		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		BasicStorehouse basicStorehouse = (BasicStorehouse) mapper.readValue(storehouseJson, BasicStorehouse.class);
		basicStorehouse.setDelFlag(1);

		String cfbm = "";//仓房编码
		String ajbm = "";//廒间编码

		if (!"".equals(basicStorehouse.getStorehouseCode())) {
			cfbm = BusinessUtils.setBZcfandygbm(dwbm, basicStorehouse.getStorehouseCode(), "0", orgId+"");
			ajbm = BusinessUtils.setBZajbm(cfbm, basicStorehouse.getStorehouseType()+"", "0");
			basicStorehouse.setCfbm(cfbm.trim());
			basicStorehouse.setAjbm(ajbm.trim());
		}
		if (basicStorehouse.getStorehouseId() == null) {
			String uuid = UUID.randomUUID().toString();
			//String uuid = UUID.randomUUID().toString().replaceAll("-", "");
			basicStorehouse.setUuid(uuid);

			if(!ParameterUtil.isequal(basicStorehouse.getLibraryType(),"1")){
				basicStorehouse.setDepotId(orgId);
			}
			basicStorehouse.setOrgId(orgId);
			basicStorehouse.setCreatename(userName);
			basicStorehouse.setCreatedate(date);
			basic.insertSelective(basicStorehouse);
		} else {
			if(!ParameterUtil.isequal(basicStorehouse.getLibraryType(),"1")){
				basicStorehouse.setDutystoreman(null);
			}
			basicStorehouse.setUpdatename(userName);
			basicStorehouse.setUpdatedate(date);
			basic.updateByPrimaryKey(basicStorehouse);
		}

		//更新货位数据
		warehouseService.saveOrUpdate(basicStorehouse, warehouseJson);

		//更新单位表仓房数、油罐数、有效仓容、有效罐容
		editOrgChNum(orgId);
		modelMap.put("status", "success");

		return modelMap;
	}

	@Override
	public void editOrgChNum(Integer orgId) throws Exception {
		//查询当前单位仓房数
		BasicStorehouseExample cf_example = new BasicStorehouseExample();
		BasicStorehouseExample.Criteria cf_criteria = cf_example.createCriteria();
		cf_criteria.andOrgIdEqualTo(orgId);
		cf_criteria.andLibraryTypeEqualTo("0");
		cf_criteria.andDelFlagEqualTo(1);
		int ch_count = basic.countByExample(cf_example);
		
		//查询当前单位的油罐数
		BasicTankExample yg_example = new BasicTankExample();
		BasicTankExample.Criteria yg_criteria = yg_example.createCriteria();
		yg_criteria.andOrgIdEqualTo(orgId);
		yg_criteria.andLibraryTypeEqualTo("0");
		yg_criteria.andDelFlagEqualTo(1);
		int yg_count = basicTankMapper.countByExample(yg_example);

		//查询当前单位仓房总实际容量和油罐总实际容量
		Map<String, Object> rl_map = basic.queryCrAndgw(orgId);

		//更新当前单位下仓房数、油罐数数、有效仓房、有效罐容
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("orgId", orgId);
		map.put("cf_count", ch_count);
		map.put("yg_count", yg_count);
		map.put("cr_count", rl_map.get("cr_count"));
		map.put("gr_count", rl_map.get("gr_count"));

		basic.editOrgCfAndHwNum(map);
	}

	@Override
	public Map<String, Object> removeStorehouse(Integer id) throws Exception {
		Map<String, Object> modelMap = new HashMap<String, Object>();

		// 先查询到这条数据
		BasicStorehouse basicStorehouse = basic.selectByPrimaryKey(id);

		// 如果当前是在用，则改成0，如果当前是弃用则改成1
		Integer delFlag = basicStorehouse.getDelFlag();
		if (delFlag == 1 || "1".equals(delFlag)) {
			delFlag = 0;
			/* 要是启用的话就同时启用掉这个仓房下的货位 */
			warehouseService.deletEexample(basicStorehouse.getStorehouseId());
		} else if (delFlag == 0 || "0".equals(delFlag)) {
			delFlag = 1;
		}
		basicStorehouse.setDelFlag(delFlag);
		basic.updateByPrimaryKey(basicStorehouse);

		//更新单位表仓房数、油罐数、有效仓容、有效罐容
		editOrgChNum(basicStorehouse.getOrgId());

		modelMap.put("status", "success");

		return modelMap;
	}

	@Override
	public PageInfo<BasicStorehouse> findStoreHousePageList(Integer pageNum, Integer pageSize, Integer houseNo, String userInfo,
			String userRoleList) throws Exception {
		PageInfo<BasicStorehouse> pageInfo = null;
		//得到当前登录人id
		JSONObject user = null;
		Integer orgId = null;
		Integer userId = null;
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
			orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
			userId = Integer.parseInt(user.get("userId") == null?"":user.get("userId").toString());
		}

		//取出仓储科科长，保管员角色的id    词典5417  词典5418
		//①保管员角色编号5417  找到枚举编码就是对应保管员角色的id
		BasicEnum basicEnum=null;
		String wareDpt=null;
		try {
			//取出仓储科科长
			basicEnum = enumMapper.selectByPrimaryKey(5418);
			wareDpt = basicEnum.getEnumcode();//仓储科长角色对应的枚举编码
		} catch (Exception e) {
			e.printStackTrace();
		}

		BasicStorehouseExample storehouseExample = new BasicStorehouseExample();
		BasicStorehouseExample.Criteria Cria = storehouseExample.createCriteria();
		List<BasicStorehouse> list = null;
		Cria.andDelFlagEqualTo(1); // 查询正常数据
		if (null != orgId && !"".equals(orgId)) {
			Cria.andOrgIdEqualTo(orgId);
		}
		//根据保管员id和仓储科id判断当前登录人属于这两中角色的什么角色
		//①登录人包含仓储科角色(查询出所有的仓房)
		if(userRoleList.contains(wareDpt)){
			list = basic.selectByExample(storehouseExample);
		} else {
			//②登录人包含保管员角色(查询出自己管理的仓房)
			BasicKeeperExample keeperExample = new BasicKeeperExample();
			BasicKeeperExample.Criteria keeperCriteria=keeperExample.createCriteria();
			if (null != userId && !"".equals(userId)) {
				keeperCriteria.andUserIdEqualTo(userId);
			}
			//查询未删除的标记'删除状态(1.正常，0.删除)'
			keeperCriteria.andDelFlagEqualTo(1);
			List<BasicKeeper> keeperList = keeperService.findCondition(keeperExample);
			if(keeperList.size()>0 && keeperList != null){
				BasicKeeperHouseExample keeperHouseExample = new BasicKeeperHouseExample();
				BasicKeeperHouseExample.Criteria Criteria=keeperHouseExample.createCriteria();
				Criteria.andKeeperIdEqualTo(Integer.valueOf(keeperList.get(0).getId()));
				if (null != orgId && !"".equals(orgId)) {
					Criteria.andOrgIdEqualTo(orgId);
				}
				//2.1查询出该保管员管理的哪些仓房
				List<BasicKeeperHouse> keeperHouseList = keeperHouseService.findCondition(keeperHouseExample);

				// 仓房集合
				List<Integer> houseIdList = new ArrayList<Integer>();

				if(keeperHouseList.size()>0 && keeperHouseList != null){
					for (BasicKeeperHouse be : keeperHouseList) {
						String houseId = be.getHouseId();
						//YG开头是是油罐，否则是粮食仓房
						if (houseId.length() > 2 && "YG".equals(houseId.substring(0, 2))) {
							
						} else {
							houseIdList.add(Integer.parseInt(houseId));
						}
					}
					//前台传过来houseId 说明是按照仓房编号查询的筛选条件  否则是查询所有的保管员所管理的仓
					if(houseNo !=null && houseIdList.contains(houseNo) ){
						houseIdList.clear();//先清空list里面的仓  添加查询的仓
						houseIdList.add(houseNo);
					}else if(houseNo !=null && !houseIdList.contains(houseNo)){
						houseIdList.clear();//先清空list里面的仓  添加查询的仓
						houseIdList.add(0);//这里随便写的数字  为了防止按条件查询到不想要的数据
					}
				}

				if (pageNum != null && pageSize != null) {
					PageHelper.startPage(pageNum, pageSize);
				}

				//2.2取出这些仓房信息
				if(houseIdList.size()>0 && houseIdList != null){
					Cria.andStorehouseIdIn(houseIdList);
				}else{
					houseIdList.add(0);//这里随便写的数字  为了防止按条件查询到不想要的数据
					Cria.andStorehouseIdIn(houseIdList);
				}

				list = basic.selectByExample(storehouseExample);
			}
		}

		pageInfo = new PageInfo<BasicStorehouse>(list);
		return pageInfo;
	}

	@Override
	public Map<String, Object> findStorehouseListByLoginUser(String userInfo) throws Exception {
		JSONObject user = null;
		Integer orgId = null;
		Integer userId = null;
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
			orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
			userId = Integer.parseInt(user.get("userId") == null?"":user.get("userId").toString());
		}
		BasicKeeperExample keeperExample = new BasicKeeperExample();
		BasicKeeperExample.Criteria keeperCriteria = keeperExample.createCriteria();

		Map<String, Object> paramMap = new HashMap<String, Object>();
		List<BasicStorehouse> storehouseList=new ArrayList<BasicStorehouse>();
		List<BasicTank> tankList=new ArrayList<BasicTank>();

		//5417保管员角色编号Id  enumCode对应的就是保管员的id
		//basicEnum = enumService.findByParmeryKeyEnum(5417);
		//keeperId=basicEnum.getEnumcode();//保管员角色对应的id
		//查询当前正常用户的角色为保管员
		keeperCriteria.andUserIdEqualTo(userId);
		keeperCriteria.andDelFlagEqualTo(1);
		//keeperCriteria.andPostTypeEqualTo(Integer.valueOf(keeperId));
		List<BasicKeeper> keeperList = keeperService.findCondition(keeperExample);
		if(keeperList.size()>0 && keeperList != null){
			BasicKeeperHouseExample keeperHouseExample = new BasicKeeperHouseExample();
			BasicKeeperHouseExample.Criteria Criteria=keeperHouseExample.createCriteria();
			Criteria.andKeeperIdEqualTo(Integer.valueOf(keeperList.get(0).getId()));
			Criteria.andOrgIdEqualTo(orgId);
			//2.1查询出该保管员管理的哪些仓房
			List<BasicKeeperHouse> keeperHouseList = keeperHouseService.findCondition(keeperHouseExample);

			BasicStorehouseExample example = new BasicStorehouseExample();
			Criteria Cri = example.createCriteria();

			BasicStorehouse basicStorehouse=null;
			BasicTank basicTank=null;
			for (BasicKeeperHouse be : keeperHouseList) {
				String houseId = be.getHouseId();

				//YG开头是是油罐，否则是粮食仓房
				if (houseId.length() > 2 && "YG".equals(houseId.substring(0, 2))) {
					basicTank = tankService.findByBasicBankPrimary(houseId);
					tankList.add(basicTank);
				} else {
					Cri.andStorehouseIdEqualTo(Integer.parseInt(houseId));
					try{
						//2.2根据仓房id查询对应的仓房信息
						basicStorehouse = this.findByPrimary(Integer.parseInt(houseId));
						storehouseList.add(basicStorehouse);
					}catch (Exception ex){
						ex.printStackTrace();
					}
				}
			}
			paramMap.put("tankList",tankList);
			paramMap.put("storehouseList",storehouseList);
		}

		return paramMap;
	}

	@Override
	public String findStoreCode(HttpServletRequest request, String storehouseCode, Integer storeId, String libraryType) throws Exception {
		String codeFlag = "";
		Integer orgId = (Integer) request.getSession().getAttribute("orgId");
		BasicStorehouseExample storehouseExample = new BasicStorehouseExample();
		BasicStorehouseExample.Criteria cria = storehouseExample.createCriteria();
		cria.andOrgIdEqualTo(orgId);
		cria.andStorehouseIdNotEqualTo(storeId);
		//cria.andLibraryTypeEqualTo(libraryType);
		cria.andStorehouseCodeEqualTo(storehouseCode);
		List<BasicStorehouse> list = basic.selectByExample(storehouseExample);
		if (null != list && list.size() > 0) {
			codeFlag = "0";
		} else {
			codeFlag = "1";
		}
		return codeFlag;
	}

	@Override
	public List<BasicStorehouse> getAgentList(Integer depotId, Integer orgId, String libraryType) throws Exception {
		BasicStorehouseExample example = new BasicStorehouseExample();
		Criteria Criteria = example.createCriteria();
		Criteria.andDepotIdEqualTo(depotId);
		Criteria.andDelFlagEqualTo(1);
		Criteria.andLibraryTypeEqualTo(libraryType);
		Criteria.andOrgIdEqualTo(orgId);

		example.setOrderByClause(" storehouse_id asc");
		List<BasicStorehouse> list = basic.selectByExample(example);
		return list;
	}

}
