package com.chinaitop.depot.basic.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import com.chinaitop.depot.basic.model.*;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.mapper.BasicTankMapper;
import com.chinaitop.depot.basic.mapper.BasicWarehouseMapper;
import com.chinaitop.depot.basic.model.BasicWarehouseExample.Criteria;
import com.chinaitop.depot.basic.service.BasicStorehouseService;
import com.chinaitop.depot.basic.service.BasicTankService;
import com.chinaitop.depot.basic.service.BasicWarehouseService;
import com.chinaitop.depot.utils.BusinessUtils;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

@Service
public class BasicWarehouseServiceImpl implements BasicWarehouseService {

	@Resource
	private BasicWarehouseMapper mapper;

	@Resource
	private BasicStorehouseService basicStorehouseService;
	
	@Resource
	private BasicTankMapper basicTankMapper;;
	
	@Override
	public void add(BasicWarehouse basicWarehouse) {
		mapper.insertSelective(basicWarehouse);
	}

	@Override
	public void update(BasicWarehouse basicWarehouse) {
		mapper.updateByPrimaryKeySelective(basicWarehouse);
	}

	@Override
	public void delete(Integer id) {
		mapper.deleteByPrimaryKey(id);
	}

	@Override
	public BasicWarehouse findByParmaryKey(Integer id) {
		return mapper.selectByPrimaryKey(id);
	}

	@Override
	public List<BasicWarehouse> findCondition(BasicWarehouseExample example) {
		return mapper.selectByExample(example);
	}
	
	@Override
	public List<BasicWarehouse> getList(Integer storehouseId, Integer warehouseId, 
			String warehouseName, Integer orgId, String libraryType, Integer delFlag) {
		List<BasicWarehouse> list = null;
		Map<String, Object> params = new HashMap<String, Object>();
		try {
			// 拼接查询条件
			//货位名称
			if (!"".equals(warehouseName) && warehouseName != null) {
				params.put("warehouseName", "'%"+warehouseName+"%'");
			}
			//仓房ID
			if (null != storehouseId && !"".equals(storehouseId)) {
				params.put("storehouseId", storehouseId);
			}
			//货位ID
			if (null != warehouseId && !"".equals(warehouseId) ){
				params.put("warehouseId", warehouseId);
			}
			//代储
			if (null != libraryType) {
				//如果是3则说明是空仓过滤
				if(ParameterUtil.isequal("3",libraryType)){
					//1说明是空仓
					params.put("crkStatus", 1);
				}else {
					params.put("libraryType", libraryType);
				}
			}

			/* 单位 */
			if (null != orgId) {
				params.put("orgId", orgId);
			}

			// 值为1是没删除的数据(在用),0是已删除的数据(弃用)
			if (null != delFlag) {
				params.put("delFlag", delFlag);
			}
			list = mapper.getList(params);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}

	@Override
	public Map<String, Object> getWarehouse(Integer unitId, Integer storehouseId, String libraryType) {
		Map<String, Object> map = new HashMap<String, Object>();
		BasicWarehouseExample example = new BasicWarehouseExample();
		Criteria criteria = example.createCriteria();
		List<BasicWarehouse> warelist = mapper.selectByExample(example);  //获取所有货位，与仓房和库点无关
		Map<Integer, Object> wareMap = new HashMap<Integer, Object>();
		if (null != warelist && warelist.size() != 0) {
			for (BasicWarehouse ware : warelist) {
				wareMap.put(ware.getWarehouseId(), ware);
			}
		} else {
			warelist = new ArrayList<BasicWarehouse>();
		}
		map.put("wares", wareMap); // 数据列表页面转换对象
		
		if (unitId != null && !"".equals(unitId)) {
			criteria.andOrgIdEqualTo(unitId);
		}
		if (storehouseId != null && !"".equals(storehouseId)) {
			criteria.andStorehouseIdEqualTo(storehouseId);
		}
		if (libraryType != null && !"".equals(libraryType)) {
			criteria.andLibraryTypeEqualTo(libraryType);
		}
		criteria.andDelFlagEqualTo(1);
		example.setOrderByClause("warehouse_id ASC");
		List<BasicWarehouse> list = mapper.selectByExample(example); //获取库点下的某个仓房中的的货位
		map.put("wareList", list); //货位下拉列表集合
		
		
		//特殊需要，把货位ID也转换成string类型的数据
		//1、建立一个放货位的数据大对象，并且在建立一个存放货位数据对象的list
		BasicHouseTank house_tank = null;
		List<BasicHouseTank> ware_list = new ArrayList<BasicHouseTank>();

		//2、放入货位信息
		for (BasicWarehouse ware : list) {
			house_tank = new BasicHouseTank();
			house_tank.setCode(ware.getWarehouseCode());
			house_tank.setId(ware.getWarehouseId() + "");
			house_tank.setName(ware.getWarehouseName());
			ware_list.add(house_tank);
		}

		//3、放入要返回的map对象
		map.put("ware_list", ware_list);
		
		return map;
	}

	@Override
	public void deletEexample(Integer storehouseId) {
		//条件
		BasicWarehouseExample example = new BasicWarehouseExample();
		Criteria criteria = example.createCriteria();
		criteria.andStorehouseIdEqualTo(storehouseId);
		criteria.andDelFlagEqualTo(1);

		//要修改的值
		BasicWarehouse warehouse = new BasicWarehouse();
		warehouse.setDelFlag(0);
		mapper.updateByExampleSelective(warehouse, example);
	}

	@Override
	public PageInfo<BasicWarehouse> getPageList(Integer pageNum, Integer pageSize, Integer storehouseId,
			Integer warehouseId, String warehouseName, Integer graindepotId, String libraryType, Integer delFlag)
			throws Exception {
		PageInfo<BasicWarehouse> pageInfo = null;
		List<BasicWarehouse> list = null;
		try {
			if (null != pageNum && null != pageSize) {
				PageHelper.startPage(pageNum, pageSize);
			}
			list = this.getList(storehouseId, warehouseId, warehouseName, graindepotId, libraryType, delFlag);
			pageInfo = new PageInfo<BasicWarehouse>(list);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return pageInfo;
	}

	@Override
	public Map<String, Object> updateOrSave(String warehouseJson, String userInfo) throws Exception {
		Map<String, Object> map = new HashMap<String, Object>();
		JSONObject user = null;
		Integer orgId = null;
        String userName = "";
        if (null != userInfo && !"".equals(userInfo)) {
        	user = JSONObject.parseObject(userInfo);
        	orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
        	userName = user.get("username") == null?"":user.get("username").toString();
        }
		Date date = new Date();
		// JSON字符串转对象
        ObjectMapper mapperWare = new ObjectMapper();

		BasicWarehouse warehouse = (BasicWarehouse)mapperWare.readValue(warehouseJson, BasicWarehouse.class);
		String hwdm = "";
		if (!"".equals(warehouse.getWarehouseCode())) {
			BasicStorehouse storehouse = basicStorehouseService.findByPrimary(warehouse.getStorehouseId());
			hwdm = BusinessUtils.setBZhwbm(storehouse.getAjbm(), warehouse.getWarehouseCode());
			warehouse.setHwdm(hwdm);
		}
		if (warehouse.getWarehouseId() != null) {
			warehouse.setUpdatename(userName);
			warehouse.setUpdatedate(date);
			mapper.updateByPrimaryKeySelective(warehouse);
		} else {
			warehouse.setWarehouseStatus(2963); //添加时默认货位状态为空仓
			warehouse.setOrgId(orgId);
			if(!ParameterUtil.isequal(warehouse.getLibraryType(),"1")){
				warehouse.setDepotId(orgId);
			}
			warehouse.setCreatename(userName);
			warehouse.setCreatedate(date);
			warehouse.setDelFlag(1);
			mapper.insertSelective(warehouse);
		}
		map.put("status", "success");

		return map;
	}

	@Override
	public Map<String, Object> qyYNoN(Integer id, Integer storehouseId) throws Exception {
		Map<String, Object> map = new HashMap<String, Object>();

    	//先查询这条仓房数据,进行判断
    	BasicWarehouse basicWarehouse = mapper.selectByPrimaryKey(id);
    	//先查询这条货位数据，通过判断如果为弃用状态则根据仓房状态进行修改,若仓房状态为应用状态,则不对货位状态进行修改
    	BasicStorehouse basicStorehouse = basicStorehouseService.findByPrimary(storehouseId);
    	Integer delFlag = basicWarehouse.getDelFlag();
    	if (delFlag == 1 || "1".equals(delFlag)) {
			delFlag = 0;
		} else if (delFlag == 0 || "0".equals(delFlag)) {
			delFlag = 1;
			//判断为弃用改应用状态,则进行应用
			basicStorehouse.setDelFlag(delFlag);
			basicStorehouseService.updateBasicStorehouse(basicStorehouse);
		}

    	basicWarehouse.setDelFlag(delFlag);
		if (id != null) {
			mapper.updateByPrimaryKeySelective(basicWarehouse);;
		}
		map.put("status", "success");
		
		return map;
	}

	@Override
	public String findWareCode(HttpServletRequest request, Integer storehouseId, String warehouseCode, Integer wareId,
			String libraryType) throws Exception {
		String codeFlag = "";
		List<BasicWarehouse> list = null;
		Integer orgId = (Integer) request.getSession().getAttribute("orgId");
		BasicWarehouseExample warehouseExample = new BasicWarehouseExample();
		Criteria cria = warehouseExample.createCriteria();
		cria.andStorehouseIdEqualTo(storehouseId);
		cria.andWarehouseCodeEqualTo(warehouseCode);
		cria.andOrgIdEqualTo(orgId);
		cria.andWarehouseIdNotEqualTo(wareId);
		//cria.andLibraryTypeEqualTo(libraryType);

		list = mapper.selectByExample(warehouseExample);
		if (null != list && list.size() > 0) {
			codeFlag = "0";
		} else {
			codeFlag = "1";
		}

		return codeFlag;
	}

	@Override
	public void updateWarehouseCrkStatus(String wareId, String crk_status) throws Exception {
		if (wareId.length() > 2 && "YG".equals(wareId.substring(0, 2))) {
			BasicTank tank = new BasicTank();
			tank.setCrkStatus(crk_status);
			tank.setId(wareId);
			//修改油管出入口状态
			basicTankMapper.updateByPrimaryKeySelective(tank);
		} else {
			BasicWarehouse warehouse = new BasicWarehouse();
			warehouse.setWarehouseId(Integer.parseInt(wareId));
			warehouse.setCrkStatus(crk_status);

			//修改数据
			mapper.updateByPrimaryKeySelective(warehouse);
		}
		
	}

	@Override
	public void saveOrUpdate(BasicStorehouse storehouse, String warehouseJson) throws Exception {
		ObjectMapper mappers = new ObjectMapper();
		List<BasicWarehouse> list = mappers.readValue(warehouseJson, new TypeReference<List<BasicWarehouse>>(){});

		String hwdm = "";
		for (BasicWarehouse warehouse : list) {
			if (!"".equals(warehouse.getWarehouseCode())) {
				hwdm = BusinessUtils.setBZhwbm(storehouse.getAjbm(), warehouse.getWarehouseCode());
				warehouse.setHwdm(hwdm);
			}
			if (null == warehouse.getWarehouseId()) {//没有货位ID那就是新增
				warehouse.setStorehouseId(storehouse.getStorehouseId());
				warehouse.setCreatename(storehouse.getCreatename());
				warehouse.setCreatedate(storehouse.getCreatedate());
				warehouse.setOrgId(storehouse.getOrgId());
				if (ParameterUtil.isequal(warehouse.getLibraryType(),"1")) {
					warehouse.setDepotId(storehouse.getDepotId());
					warehouse.setDepotName(storehouse.getDepotName());
				}
				mapper.insertSelective(warehouse);
			} else {
				warehouse.setUpdatename(storehouse.getUpdatename());
				warehouse.setUpdatedate(storehouse.getUpdatedate());
				mapper.updateByPrimaryKeySelective(warehouse);
			}
		}
	}

	@Override
	public BasicWarehouseVO getWareDataByHwbm(String hwbm) {
		return mapper.getWareDataByHwbm(hwbm);
	}
}
