package com.chinaitop.depot.unissoft.webservice;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.ObjectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.mapper.BasicTankHwMapper;
import com.chinaitop.depot.basic.mapper.BasicTankMapper;
import com.chinaitop.depot.basic.mapper.BasicWarehouseMapper;
import com.chinaitop.depot.basic.model.BasicTank;
import com.chinaitop.depot.basic.model.BasicWarehouse;
import com.chinaitop.utils.ParameterUtil;

@Service
public class BasicWarehouseWebServiceImpl implements BasicWarehouseWebService {
	final static Logger logger = LoggerFactory.getLogger(BasicWarehouse.class);

	@Resource
	private BasicWarehouseMapper warehouseMapper;

	@Resource
	private BasicTankMapper tankMapper;
	
	@Resource
	private BasicTankHwMapper tankHwMapper;

	@Override
	@SuppressWarnings("all")
	public synchronized String uploadCfStatus(String json)  throws Exception {
		Map<String, Object> result = new HashMap<String, Object>();

		String date = ParameterUtil.getSysDateTime();
//		logger.info("时间："+date+"，出入库状态回写接口，出入库系统调用，接收参数是："+json);
		if (null != json) {
			/* 将获取到的json串转换成json数组 */
			JSONArray json_array = JSONArray.parseArray(json);
			JSONObject json_object = null;
			
			/* 遍历json数组，获取数组中值 */
			for (Iterator iterator = json_array.iterator(); iterator.hasNext();) {
				
				/* 获取json对象 */
				json_object = (JSONObject) iterator.next();
				if (json_object == null) {
					continue;
				}

				//货位ID
				String wareId = ObjectUtils.toString(json_object.get("hwId"), "");
				//单位ID
				String orgId = ObjectUtils.toString(json_object.get("org_Id"), "");
				//货位状态值1空仓,2入库中,3封仓,4出库中
				String status = ObjectUtils.toString(json_object.get("status"), "");
				//业务时间
				String ywsj = ObjectUtils.toString(json_object.get("cjsj"), "");

				//修改数据
				Map<String, Object> param = new HashMap<String, Object>();
				param.put("orgId", Integer.parseInt(orgId));
				param.put("status", status);
				param.put("ywsj", ywsj);
				if (wareId.indexOf("YG") > -1) {
					param.put("wareId", wareId);
					//回写粮食仓房的
					yg(param);
				} else {
					param.put("wareId", Integer.parseInt(wareId));
					//回写油罐的
					ls(param);
				}

			}
		}
		result.put("success", "true");
		result.put("msg", "上传成功");
		logger.info("时间："+date+"，出入库状态回写接口，出入库系统调用，调用成功！");

		JSONArray jsonArray = new JSONArray();
		jsonArray.add(result);
		return jsonArray.toString();
	}

	public void ls(Map<String, Object> param) {
		//查询当前货位状态
		BasicWarehouse warehouse = warehouseMapper.selectByPrimaryKey(Integer.parseInt(param.get("wareId")+""));
		if (null != warehouse && "3".equals(warehouse.getCrkStatus()) && !ParameterUtil.isequal(warehouse.getCrkStatus(),param.get("status"))) {
			//查询封仓时间
			List<Map<String, String>> map = warehouseMapper.queryFcdate(param);
			if (null != map && map.size() > 0) {
				String fcsj = map.get(0).get("filling_time")+" 00:00:00";
				//判断封仓时间在不在业务时间之前
				if (ParameterUtil.string2datetime(fcsj).after(ParameterUtil.string2datetime(param.get("ywsj")+""))) {
					//continue;
				} else {
					//回写状态
					warehouseMapper.updateWareStatus(param);
				}
			}
		} else {
			//回写状态
			warehouseMapper.updateWareStatus(param);
		}
	}

	public void yg(Map<String, Object> param) {
		//查询当前货位状态
		BasicTank warehouse = tankMapper.selectByPrimaryKey(param.get("wareId")+"");
		if (null != warehouse && "3".equals(warehouse.getCrkStatus()) && !ParameterUtil.isequal(warehouse.getCrkStatus(),param.get("status"))) {
			//查询封仓时间
			List<Map<String, String>> map = warehouseMapper.queryFcdate(param);
			if (null != map && map.size() > 0) {
				String fcsj = map.get(0).get("filling_time")+" 00:00:00";
				//判断封仓时间在不在业务时间之前
				if (ParameterUtil.string2datetime(fcsj).after(ParameterUtil.string2datetime(param.get("ywsj")+""))) {
					//continue;
				} else {
					//回写状态
					tankMapper.updateTankStatus(param);
					tankHwMapper.updateTankStatus(param);
				}
			} else {//是等于3，但是也没有封仓的数据，无法进行判断
				//回写状态
				//tankMapper.updateTankStatus(param);
				//tankHwMapper.updateTankStatus(param);
			}
		} else {
			//回写状态
			tankMapper.updateTankStatus(param);
			tankHwMapper.updateTankStatus(param);
		}
	}

}
